# copyright 2003-2011 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This file is part of logilab-common.
#
# logilab-common is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option) any
# later version.
#
# logilab-common is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with logilab-common.  If not, see <http://www.gnu.org/licenses/>.
from logilab.common.testlib import TestCase, unittest_main

from logilab.common.tasksqueue import Task, PrioritizedTasksQueue, LOW, MEDIUM, HIGH


class TaskTC(TestCase):
    def test_eq(self):
        self.assertFalse(Task("t1") == Task("t2"))
        self.assertTrue(Task("t1") == Task("t1"))

    def test_cmp(self):
        self.assertTrue(Task("t1", LOW) < Task("t2", MEDIUM))
        self.assertFalse(Task("t1", LOW) > Task("t2", MEDIUM))
        self.assertTrue(Task("t1", HIGH) > Task("t2", MEDIUM))
        self.assertFalse(Task("t1", HIGH) < Task("t2", MEDIUM))


class PrioritizedTasksQueueTC(TestCase):
    def test_priority(self):
        queue = PrioritizedTasksQueue()
        queue.put(Task("t1"))
        queue.put(Task("t2", MEDIUM))
        queue.put(Task("t3", HIGH))
        queue.put(Task("t4", LOW))
        self.assertEqual(queue.get().id, "t3")
        self.assertEqual(queue.get().id, "t2")
        self.assertEqual(queue.get().id, "t1")
        self.assertEqual(queue.get().id, "t4")

    def test_remove_equivalent(self):
        queue = PrioritizedTasksQueue()
        queue.put(Task("t1"))
        queue.put(Task("t2", MEDIUM))
        queue.put(Task("t1", HIGH))
        queue.put(Task("t3", MEDIUM))
        queue.put(Task("t2", MEDIUM))
        self.assertEqual(queue.qsize(), 3)
        self.assertEqual(queue.get().id, "t1")
        self.assertEqual(queue.get().id, "t2")
        self.assertEqual(queue.get().id, "t3")
        self.assertEqual(queue.qsize(), 0)

    def test_remove(self):
        queue = PrioritizedTasksQueue()
        queue.put(Task("t1"))
        queue.put(Task("t2"))
        queue.put(Task("t3"))
        queue.remove("t2")
        self.assertEqual([t.id for t in queue], ["t3", "t1"])
        self.assertRaises(ValueError, queue.remove, "t4")


if __name__ == "__main__":
    unittest_main()



# copyright 2003-2011 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This file is part of logilab-common.
#
# logilab-common is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option) any
# later version.
#
# logilab-common is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with logilab-common.  If not, see <http://www.gnu.org/licenses/>.
"""unit tests utilities for ureports
"""


import sys
from io import StringIO

from logilab.common.ureports.nodes import (
    Section,
    Text,
    Table,
    VerbatimText,
    List,
    Paragraph,
    Link,
)

buffers = [StringIO]
if sys.version_info < (3, 0):
    from cStringIO import StringIO as cStringIO
    from StringIO import StringIO as pStringIO

    buffers += [cStringIO, pStringIO]


class WriterTC:
    def _test_output(self, test_id, layout, msg=None):
        for buffercls in buffers:
            buffer = buffercls()
            self.writer.format(layout, buffer)
            got = buffer.getvalue()
            expected = getattr(self, test_id)
            try:
                self.assertMultiLineEqual(got, expected)
            except Exception:
                print("**** using a %s" % buffer.__class__)
                print("**** got for %s" % test_id)
                print(got)
                print("**** while expected")
                print(expected)
                print("****")
                raise

    def test_section(self):
        layout = Section("Section title", "Section's description.\nBlabla bla")
        self._test_output("section_base", layout)
        layout.append(Section("Subsection", "Sub section description"))
        self._test_output("section_nested", layout)

    def test_verbatim(self):
        layout = VerbatimText("blablabla")
        self._test_output("verbatim_base", layout)

    def test_list(self):
        layout = List(children=("item1", "item2", "item3", "item4"))
        self._test_output("list_base", layout)

    def test_nested_list(self):
        layout = List(
            children=(Paragraph(("blabla", List(children=("1", "2", "3")))), "an other point")
        )
        self._test_output("nested_list", layout)

    def test_table(self):
        layout = Table(cols=2, children=("head1", "head2", "cell1", "cell2"))
        self._test_output("table_base", layout)

    def test_field_table(self):
        table = Table(cols=2, klass="field", id="mytable")
        for field, value in (("f1", "v1"), ("f22", "v22"), ("f333", "v333")):
            table.append(Text(field))
            table.append(Text(value))
        self._test_output("field_table", table)

    def test_advanced_table(self):
        table = Table(cols=2, klass="whatever", id="mytable", rheaders=1)
        for field, value in (("field", "value"), ("f1", "v1"), ("f22", "v22"), ("f333", "v333")):
            table.append(Text(field))
            table.append(Text(value))
        table.append(Link("http://www.perdu.com", "toi perdu ?"))
        table.append(Text(""))
        self._test_output("advanced_table", table)



from logilab.common.registry import RegistrableObject, RegistrableInstance, yes


class MyRegistrableInstance(RegistrableInstance):
    __regid__ = "appobject3"
    __select__ = yes()
    __registry__ = "zereg"


instance = MyRegistrableInstance(__module__=__name__)



from data.module import YO, YOUPI
import data


class Specialization(YOUPI, YO):
    pass


class Metaclass(type):
    pass


class Interface:
    pass


class MyIFace(Interface):
    pass


class AnotherIFace(Interface):
    pass


class MyException(Exception):
    pass


class MyError(MyException):
    pass


class AbstractClass:
    def to_override(self, whatever):
        raise NotImplementedError()

    def return_something(self, param):
        if param:
            return "toto"
        return


class Concrete0:
    __implements__ = MyIFace


class Concrete1:
    __implements__ = MyIFace, AnotherIFace


class Concrete2:
    __implements__ = (MyIFace, AnotherIFace)


class Concrete23(Concrete1):
    pass


del YO.member

del YO
[SYN1, SYN2] = Concrete0, Concrete1
assert "1"
b = 1 | 2 & 3 ^ 8
exec("c = 3")
exec("c = 3", {}, {})


def raise_string(a=2, *args, **kwargs):
    raise "pas glop"
    raise Exception("yo")
    yield "coucou"


a = b + 2
c = b * 2
c = b / 2
c = b // 2
c = b - 2
c = b % 2
c = b**2
c = b << 2
c = b >> 2
c = ~b

c = not b

d = [c]
e = d[:]
e = d[a:b:c]

raise_string(*args, **kwargs)

print >> stream, "bonjour"
print >> stream, "salut",


def make_class(any, base=data.module.YO, *args, **kwargs):
    """check base is correctly resolved to Concrete0"""

    class Aaaa(base):
        """dynamic class"""

    return Aaaa



# copyright 2003-2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This file is part of Logilab-Common.
#
# Logilab-Common is free software: you can redistribute it and/or modify it under the
# terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# Logilab-Common is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with Logilab-Common.  If not, see <http://www.gnu.org/licenses/>.
"""unit tests for selectors mechanism"""

import gc
import logging
import sys
from contextlib import contextmanager
import warnings

from logilab.common.testlib import (
    TestCase,
    unittest_main,
)

from logilab.common.registry import (
    Predicate,
    AndPredicate,
    OrPredicate,
    wrap_predicates,
    RegistryStore,
    RegistrableInstance,
)

logging.basicConfig(level=logging.ERROR)


class _1_(Predicate):
    def __call__(self, *args, **kwargs):
        return 1


class _0_(Predicate):
    def __call__(self, *args, **kwargs):
        return 0


def _2_(*args, **kwargs):
    return 2


class SelectorsTC(TestCase):
    def test_basic_and(self):
        selector = _1_() & _1_()
        self.assertEqual(selector(None), 2)
        selector = _1_() & _0_()
        self.assertEqual(selector(None), 0)
        selector = _0_() & _1_()
        self.assertEqual(selector(None), 0)

    def test_basic_or(self):
        selector = _1_() | _1_()
        self.assertEqual(selector(None), 1)
        selector = _1_() | _0_()
        self.assertEqual(selector(None), 1)
        selector = _0_() | _1_()
        self.assertEqual(selector(None), 1)
        selector = _0_() | _0_()
        self.assertEqual(selector(None), 0)

    def test_selector_and_function(self):
        selector = _1_() & _2_
        self.assertEqual(selector(None), 3)
        selector = _2_ & _1_()
        self.assertEqual(selector(None), 3)

    def test_three_and(self):
        selector = _1_() & _1_() & _1_()
        self.assertEqual(selector(None), 3)
        selector = _1_() & _0_() & _1_()
        self.assertEqual(selector(None), 0)
        selector = _0_() & _1_() & _1_()
        self.assertEqual(selector(None), 0)

    def test_three_or(self):
        selector = _1_() | _1_() | _1_()
        self.assertEqual(selector(None), 1)
        selector = _1_() | _0_() | _1_()
        self.assertEqual(selector(None), 1)
        selector = _0_() | _1_() | _1_()
        self.assertEqual(selector(None), 1)
        selector = _0_() | _0_() | _0_()
        self.assertEqual(selector(None), 0)

    def test_composition(self):
        selector = (_1_() & _1_()) & (_1_() & _1_())
        self.assertTrue(isinstance(selector, AndPredicate))
        self.assertEqual(len(selector.selectors), 4)
        self.assertEqual(selector(None), 4)
        selector = (_1_() & _0_()) | (_1_() & _1_())
        self.assertTrue(isinstance(selector, OrPredicate))
        self.assertEqual(len(selector.selectors), 2)
        self.assertEqual(selector(None), 2)

    def test_search_selectors(self):
        sel = _1_()
        self.assertIs(sel.search_selector(_1_), sel)
        csel = AndPredicate(sel, Predicate())
        self.assertIs(csel.search_selector(_1_), sel)
        csel = AndPredicate(Predicate(), sel)
        self.assertIs(csel.search_selector(_1_), sel)
        self.assertIs(csel.search_selector((AndPredicate, OrPredicate)), csel)
        self.assertIs(csel.search_selector((OrPredicate, AndPredicate)), csel)
        self.assertIs(csel.search_selector((_1_, _0_)), sel)
        self.assertIs(csel.search_selector((_0_, _1_)), sel)

    def test_inplace_and(self):
        selector = _1_()
        selector &= _1_()
        selector &= _1_()
        self.assertEqual(selector(None), 3)
        selector = _1_()
        selector &= _0_()
        selector &= _1_()
        self.assertEqual(selector(None), 0)
        selector = _0_()
        selector &= _1_()
        selector &= _1_()
        self.assertEqual(selector(None), 0)
        selector = _0_()
        selector &= _0_()
        selector &= _0_()
        self.assertEqual(selector(None), 0)

    def test_inplace_or(self):
        selector = _1_()
        selector |= _1_()
        selector |= _1_()
        self.assertEqual(selector(None), 1)
        selector = _1_()
        selector |= _0_()
        selector |= _1_()
        self.assertEqual(selector(None), 1)
        selector = _0_()
        selector |= _1_()
        selector |= _1_()
        self.assertEqual(selector(None), 1)
        selector = _0_()
        selector |= _0_()
        selector |= _0_()
        self.assertEqual(selector(None), 0)

    def test_wrap_selectors(self):
        class _temp_(Predicate):
            def __call__(self, *args, **kwargs):
                return 0

        del _temp_  # test weakref
        s1 = _1_() & _1_()
        s2 = _1_() & _0_()
        s3 = _0_() & _1_()
        gc.collect()
        self.count = 0

        def decorate(f, self=self):
            def wrapper(*args, **kwargs):
                self.count += 1
                return f(*args, **kwargs)

            return wrapper

        wrap_predicates(decorate)
        self.assertEqual(s1(None), 2)
        self.assertEqual(s2(None), 0)
        self.assertEqual(s3(None), 0)
        self.assertEqual(self.count, 8)


@contextmanager
def prepended_syspath(path):
    sys.path.insert(0, path)
    yield
    sys.path = sys.path[1:]


class RegistryStoreTC(TestCase):
    def test_autoload(self):
        store = RegistryStore()
        store.setdefault("zereg")
        with prepended_syspath(self.datadir):
            with warnings.catch_warnings(record=True) as warns:
                store.register_objects(
                    [self.datapath("regobjects.py"), self.datapath("regobjects2.py")]
                )
                self.assertIn(
                    "[logilab.common.registry] use register_modnames() instead",
                    [str(w.message) for w in warns],
                )
        self.assertEqual(["zereg"], list(store.keys()))
        self.assertEqual({"appobject1", "appobject2", "appobject3"}, set(store["zereg"]))

    def test_autoload_modnames(self):
        store = RegistryStore()
        store.setdefault("zereg")
        with prepended_syspath(self.datadir):
            store.register_modnames(["regobjects", "regobjects2"])
        self.assertEqual(["zereg"], list(store.keys()))
        self.assertEqual({"appobject1", "appobject2", "appobject3"}, set(store["zereg"]))


class RegistrableInstanceTC(TestCase):
    def test_instance_modulename(self):
        with warnings.catch_warnings(record=True) as warns:
            obj = RegistrableInstance()
            self.assertEqual(obj.__module__, "test_registry")
            self.assertIn(
                "instantiate RegistrableInstance with __module__=__name__",
                [str(w.message) for w in warns],
            )
        # no inheritance
        obj = RegistrableInstance(__module__=__name__)
        self.assertEqual(obj.__module__, "test_registry")
        # with inheritance from another python file
        with prepended_syspath(self.datadir):
            from regobjects2 import instance, MyRegistrableInstance

            instance2 = MyRegistrableInstance(__module__=__name__)
            self.assertEqual(instance.__module__, "regobjects2")
            self.assertEqual(instance2.__module__, "test_registry")


if __name__ == "__main__":
    unittest_main()



import unittest


class TestCase(unittest.TestCase):
    def setUp(self):
        unittest.TestCase.setUp(self)

    def tearDown(self):
        unittest.TestCase.tearDown(self)

    def testIt(self):
        self.a = 10
        self.xxx()

    def xxx(self):
        if False:
            print("a")

        if False:
            pass

        if False:
            print("rara")


if __name__ == "__main__":
    print("test2")
    unittest.main()



# copyright 2003-2011 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This file is part of logilab-common.
#
# logilab-common is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option) any
# later version.
#
# logilab-common is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with logilab-common.  If not, see <http://www.gnu.org/licenses/>.
"""
unit tests for module textutils
squeleton generated by /home/syt/cvs_work/logilab/pyreverse/py2tests.py on Sep 08 at 09:1:31

"""

# flake8: noqa: E501

import doctest
import re
from os import linesep

from logilab.common import textutils as tu
from logilab.common.testlib import TestCase, unittest_main


if linesep != "\n":
    LINE_RGX = re.compile(linesep)

    def ulines(string):
        return LINE_RGX.sub("\n", string)

else:

    def ulines(string):
        return string


class NormalizeTextTC(TestCase):
    def test_known_values(self):
        self.assertEqual(
            ulines(
                tu.normalize_text(
                    """some really malformated
        text.
With some times some veeeeeeeeeeeeeeerrrrryyyyyyyyyyyyyyyyyyy loooooooooooooooooooooong linnnnnnnnnnnes

and empty lines!
        """
                )
            ),
            """some really malformated text. With some times some
veeeeeeeeeeeeeeerrrrryyyyyyyyyyyyyyyyyyy loooooooooooooooooooooong
linnnnnnnnnnnes

and empty lines!""",
        )
        self.assertMultiLineEqual(
            ulines(
                tu.normalize_text(
                    """\
some ReST formated text
=======================
With some times some veeeeeeeeeeeeeeerrrrryyyyyyyyyyyyyyyyyyy loooooooooooooooooooooong linnnnnnnnnnnes
and normal lines!

another paragraph
        """,
                    rest=True,
                )
            ),
            """\
some ReST formated text
=======================
With some times some veeeeeeeeeeeeeeerrrrryyyyyyyyyyyyyyyyyyy
loooooooooooooooooooooong linnnnnnnnnnnes
and normal lines!

another paragraph""",
        )

    def test_nonregr_unsplitable_word(self):
        self.assertEqual(
            ulines(
                tu.normalize_text(
                    """petit complement :

http://www.plonefr.net/blog/archive/2005/10/30/tester-la-future-infrastructure-i18n
""",
                    80,
                )
            ),
            """petit complement :

http://www.plonefr.net/blog/archive/2005/10/30/tester-la-future-infrastructure-i18n""",
        )

    def test_nonregr_rest_normalize(self):
        self.assertEqual(
            ulines(
                tu.normalize_text(
                    """... Il est donc evident que tout le monde doit lire le compte-rendu de RSH et aller discuter avec les autres si c'est utile ou necessaire.
        """,
                    rest=True,
                )
            ),
            """... Il est donc evident que tout le monde doit lire le compte-rendu de RSH et
aller discuter avec les autres si c'est utile ou necessaire.""",
        )

    def test_normalize_rest_paragraph(self):
        self.assertEqual(
            ulines(tu.normalize_rest_paragraph("""**nico**: toto""")), """**nico**: toto"""
        )

    def test_normalize_rest_paragraph2(self):
        self.assertEqual(
            ulines(
                tu.normalize_rest_paragraph(
                    """.. _tdm: http://www.editions-eni.fr/Livres/Python-Les-fondamentaux-du-langage---La-programmation-pour-les-scientifiques-Table-des-matieres/.20_adaa41fb-c125-4919-aece-049601e81c8e_0_0.pdf
.. _extrait: http://www.editions-eni.fr/Livres/Python-Les-fondamentaux-du-langage---La-programmation-pour-les-scientifiques-Extrait-du-livre/.20_d6eed0be-0d36-4384-be59-2dd09e081012_0_0.pdf""",
                    indent="> ",
                )
            ),
            """> .. _tdm:
> http://www.editions-eni.fr/Livres/Python-Les-fondamentaux-du-langage---La-programmation-pour-les-scientifiques-Table-des-matieres/.20_adaa41fb-c125-4919-aece-049601e81c8e_0_0.pdf
> .. _extrait:
> http://www.editions-eni.fr/Livres/Python-Les-fondamentaux-du-langage---La-programmation-pour-les-scientifiques-Extrait-du-livre/.20_d6eed0be-0d36-4384-be59-2dd09e081012_0_0.pdf""",
        )

    def test_normalize_paragraph2(self):
        self.assertEqual(
            ulines(
                tu.normalize_paragraph(
                    """.. _tdm: http://www.editions-eni.fr/Livres/Python-Les-fondamentaux-du-langage---La-programmation-pour-les-scientifiques-Table-des-matieres/.20_adaa41fb-c125-4919-aece-049601e81c8e_0_0.pdf
.. _extrait: http://www.editions-eni.fr/Livres/Python-Les-fondamentaux-du-langage---La-programmation-pour-les-scientifiques-Extrait-du-livre/.20_d6eed0be-0d36-4384-be59-2dd09e081012_0_0.pdf""",
                    indent="> ",
                )
            ),
            """> .. _tdm:
> http://www.editions-eni.fr/Livres/Python-Les-fondamentaux-du-langage---La-programmation-pour-les-scientifiques-Table-des-matieres/.20_adaa41fb-c125-4919-aece-049601e81c8e_0_0.pdf
> .. _extrait:
> http://www.editions-eni.fr/Livres/Python-Les-fondamentaux-du-langage---La-programmation-pour-les-scientifiques-Extrait-du-livre/.20_d6eed0be-0d36-4384-be59-2dd09e081012_0_0.pdf""",
        )


class NormalizeParagraphTC(TestCase):
    def test_known_values(self):
        self.assertEqual(
            ulines(
                tu.normalize_text(
                    """This package contains test files shared by the logilab-common package. It isn't
necessary to install this package unless you want to execute or look at
the tests.""",
                    indent=" ",
                    line_len=70,
                )
            ),
            """\
 This package contains test files shared by the logilab-common
 package. It isn't necessary to install this package unless you want
 to execute or look at the tests.""",
        )


class GetCsvTC(TestCase):
    def test_known(self):
        self.assertEqual(tu.splitstrip("a, b,c "), ["a", "b", "c"])


class UnitsTC(TestCase):
    def setUp(self):
        self.units = {
            "m": 60,
            "kb": 1024,
            "mb": 1024 * 1024,
        }

    def test_empty_base(self):
        self.assertEqual(tu.apply_units("17", {}), 17)

    def test_empty_inter(self):
        def inter(value):
            return int(float(value)) * 2

        result = tu.apply_units("12.4", {}, inter=inter)
        self.assertEqual(result, 12 * 2)
        self.assertIsInstance(result, float)

    def test_empty_final(self):
        # int('12.4') raise value error
        self.assertRaises(ValueError, tu.apply_units, "12.4", {}, final=int)

    def test_empty_inter_final(self):
        result = tu.apply_units("12.4", {}, inter=float, final=int)
        self.assertEqual(result, 12)
        self.assertIsInstance(result, int)

    def test_blank_base(self):
        result = tu.apply_units(" 42  ", {}, final=int)
        self.assertEqual(result, 42)

    def test_blank_space(self):
        result = tu.apply_units(" 1 337 ", {}, final=int)
        self.assertEqual(result, 1337)

    def test_blank_coma(self):
        result = tu.apply_units(" 4,298.42 ", {})
        self.assertEqual(result, 4298.42)

    def test_blank_mixed(self):
        result = tu.apply_units("45, 317, 337", {}, final=int)
        self.assertEqual(result, 45317337)

    def test_unit_singleunit_singleletter(self):
        result = tu.apply_units("15m", self.units)
        self.assertEqual(result, 15 * self.units["m"])

    def test_unit_singleunit_multipleletter(self):
        result = tu.apply_units("47KB", self.units)
        self.assertEqual(result, 47 * self.units["kb"])

    def test_unit_singleunit_caseinsensitive(self):
        result = tu.apply_units("47kb", self.units)
        self.assertEqual(result, 47 * self.units["kb"])

    def test_unit_multipleunit(self):
        result = tu.apply_units("47KB 1.5MB", self.units)
        self.assertEqual(result, 47 * self.units["kb"] + 1.5 * self.units["mb"])

    def test_unit_with_blank(self):
        result = tu.apply_units("1 000 KB", self.units)
        self.assertEqual(result, 1000 * self.units["kb"])

    def test_unit_wrong_input(self):
        self.assertRaises(ValueError, tu.apply_units, "", self.units)
        self.assertRaises(ValueError, tu.apply_units, "wrong input", self.units)
        self.assertRaises(ValueError, tu.apply_units, "wrong13 input", self.units)
        self.assertRaises(ValueError, tu.apply_units, "wrong input42", self.units)
        with self.assertRaises(ValueError) as cm:
            tu.apply_units("42 cakes", self.units)
        self.assertIn("invalid unit cakes.", str(cm.exception))


RGX = re.compile("abcd")


class PrettyMatchTC(TestCase):
    def test_known(self):
        string = "hiuherabcdef"
        self.assertEqual(
            ulines(tu.pretty_match(RGX.search(string), string)), "hiuherabcdef\n      ^^^^"
        )

    def test_known_values_1(self):
        rgx = re.compile("(to*)")
        string = "toto"
        match = rgx.search(string)
        self.assertEqual(
            ulines(tu.pretty_match(match, string)),
            """toto
^^""",
        )

    def test_known_values_2(self):
        rgx = re.compile("(to*)")
        string = """ ... ... to to
 ... ... """
        match = rgx.search(string)
        self.assertEqual(
            ulines(tu.pretty_match(match, string)),
            """ ... ... to to
         ^^
 ... ...""",
        )


class UnquoteTC(TestCase):
    def test(self):
        self.assertEqual(tu.unquote('"toto"'), "toto")
        self.assertEqual(tu.unquote("'l'inenarrable toto'"), "l'inenarrable toto")
        self.assertEqual(tu.unquote("no quote"), "no quote")


class ColorizeAnsiTC(TestCase):
    def test_known(self):
        self.assertEqual(tu.colorize_ansi("hello", "blue", "strike"), "\x1b[9;34mhello\x1b[0m")
        self.assertEqual(
            tu.colorize_ansi("hello", style="strike, inverse"), "\x1b[9;7mhello\x1b[0m"
        )
        self.assertEqual(tu.colorize_ansi("hello", None, None), "hello")
        self.assertEqual(tu.colorize_ansi("hello", "", ""), "hello")

    def test_raise(self):
        self.assertRaises(KeyError, tu.colorize_ansi, "hello", "bleu", None)
        self.assertRaises(KeyError, tu.colorize_ansi, "hello", None, "italique")


class UnormalizeTC(TestCase):
    def test_unormalize_no_substitute(self):
        data = [
            ("\u0153nologie", "oenologie"),
            ("\u0152nologie", "OEnologie"),
            ("l\xf8to", "loto"),
            ("été", "ete"),
            ("àèùéïîôêç", "aeueiioec"),
            ("ÀÈÙÉÏÎÔÊÇ", "AEUEIIOEC"),
            ("\xa0", " "),  # NO-BREAK SPACE managed by NFKD decomposition
            ("\u0154", "R"),
            ("Pointe d\u2019Yves", "Pointe d'Yves"),
            ("Bordeaux\u2013Mérignac", "Bordeaux-Merignac"),
        ]
        for input, output in data:
            yield self.assertEqual, tu.unormalize(input), output

    def test_unormalize_substitute(self):
        self.assertEqual(tu.unormalize("ab \u8000 cd", substitute="_"), "ab _ cd")

    def test_unormalize_backward_compat(self):
        self.assertRaises(ValueError, tu.unormalize, "\u8000")
        self.assertEqual(tu.unormalize("\u8000", substitute=""), "")


def load_tests(loader, tests, ignore):
    tests.addTests(doctest.DocTestSuite(tu))
    return tests


if __name__ == "__main__":
    unittest_main()



# copyright 2003-2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This file is part of logilab-common.
#
# logilab-common is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option) any
# later version.
#
# logilab-common is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with logilab-common.  If not, see <http://www.gnu.org/licenses/>.
"""unit tests for the decorators module
"""
import types

from logilab.common.testlib import TestCase, unittest_main
from logilab.common.decorators import monkeypatch, cached, clear_cache, copy_cache, cachedproperty


class DecoratorsTC(TestCase):
    def test_monkeypatch_instance_method(self):
        class MyClass:
            pass

        @monkeypatch(MyClass)
        def meth1(self):
            return 12

        class XXX:
            @monkeypatch(MyClass)
            def meth2(self):
                return 12

        # with python3, unbound method are functions
        self.assertIsInstance(MyClass.meth1, types.FunctionType)
        self.assertIsInstance(MyClass.meth2, types.FunctionType)
        self.assertEqual(MyClass().meth1(), 12)
        self.assertEqual(MyClass().meth2(), 12)

    def test_monkeypatch_property(self):
        class MyClass:
            pass

        @monkeypatch(MyClass, methodname="prop1")
        @property
        def meth1(self):
            return 12

        self.assertIsInstance(MyClass.prop1, property)
        self.assertEqual(MyClass().prop1, 12)

    def test_monkeypatch_arbitrary_callable(self):
        class MyClass:
            pass

        class ArbitraryCallable:
            def __call__(self):
                return 12

        # ensure it complains about missing __name__
        with self.assertRaises(AttributeError) as cm:
            monkeypatch(MyClass)(ArbitraryCallable())
        self.assertTrue(
            str(cm.exception).endswith(
                "has no __name__ attribute: you should provide an explicit `methodname`"
            )
        )
        # ensure no black magic under the hood
        monkeypatch(MyClass, "foo")(ArbitraryCallable())
        self.assertTrue(callable(MyClass.foo))
        self.assertEqual(MyClass().foo(), 12)

    def test_monkeypatch_with_same_name(self):
        class MyClass:
            pass

        @monkeypatch(MyClass)
        def meth1(self):
            return 12

        self.assertEqual([attr for attr in dir(MyClass) if attr[:2] != "__"], ["meth1"])
        inst = MyClass()
        self.assertEqual(inst.meth1(), 12)

    def test_monkeypatch_with_custom_name(self):
        class MyClass:
            pass

        @monkeypatch(MyClass, "foo")
        def meth2(self, param):
            return param + 12

        self.assertEqual([attr for attr in dir(MyClass) if attr[:2] != "__"], ["foo"])
        inst = MyClass()
        self.assertEqual(inst.foo(4), 16)

    def test_cannot_cache_generator(self):
        def foo():
            yield 42

        self.assertRaises(AssertionError, cached, foo)

    def test_cached_preserves_docstrings_and_name(self):
        class Foo:
            @cached
            def foo(self):
                """what's up doc ?"""

            def bar(self, zogzog):
                """what's up doc ?"""

            bar = cached(bar, 1)

            @cached
            def quux(self, zogzog):
                """what's up doc ?"""

        self.assertEqual(Foo.foo.__doc__, """what's up doc ?""")
        self.assertEqual(Foo.foo.__name__, "foo")
        self.assertEqual(Foo.bar.__doc__, """what's up doc ?""")
        self.assertEqual(Foo.bar.__name__, "bar")
        self.assertEqual(Foo.quux.__doc__, """what's up doc ?""")
        self.assertEqual(Foo.quux.__name__, "quux")

    def test_cached_single_cache(self):
        class Foo:
            @cached(cacheattr="_foo")
            def foo(self):
                """what's up doc ?"""

        foo = Foo()
        foo.foo()
        self.assertTrue(hasattr(foo, "_foo"))
        clear_cache(foo, "foo")
        self.assertFalse(hasattr(foo, "_foo"))

    def test_cached_multi_cache(self):
        class Foo:
            @cached(cacheattr="_foo")
            def foo(self, args):
                """what's up doc ?"""

        foo = Foo()
        foo.foo(1)
        self.assertEqual(foo._foo, {(1,): None})
        clear_cache(foo, "foo")
        self.assertFalse(hasattr(foo, "_foo"))

    def test_cached_keyarg_cache(self):
        class Foo:
            @cached(cacheattr="_foo", keyarg=1)
            def foo(self, other, args):
                """what's up doc ?"""

        foo = Foo()
        foo.foo(2, 1)
        self.assertEqual(foo._foo, {2: None})
        clear_cache(foo, "foo")
        self.assertFalse(hasattr(foo, "_foo"))

    def test_cached_property(self):
        class Foo:
            @property
            @cached(cacheattr="_foo")
            def foo(self):
                """what's up doc ?"""

        foo = Foo()
        foo.foo
        self.assertEqual(foo._foo, None)
        clear_cache(foo, "foo")
        self.assertFalse(hasattr(foo, "_foo"))

    def test_copy_cache(self):
        class Foo:
            @cached(cacheattr="_foo")
            def foo(self, args):
                """what's up doc ?"""

        foo = Foo()
        foo.foo(1)
        self.assertEqual(foo._foo, {(1,): None})
        foo2 = Foo()
        self.assertFalse(hasattr(foo2, "_foo"))
        copy_cache(foo2, "foo", foo)
        self.assertEqual(foo2._foo, {(1,): None})

    def test_cachedproperty(self):
        class Foo:
            x = 0

            @cachedproperty
            def bar(self):
                self.__class__.x += 1
                return self.__class__.x

            @cachedproperty
            def quux(self):
                """some prop"""
                return 42

        foo = Foo()
        self.assertEqual(Foo.x, 0)
        self.assertFalse("bar" in foo.__dict__)
        self.assertEqual(foo.bar, 1)
        self.assertTrue("bar" in foo.__dict__)
        self.assertEqual(foo.bar, 1)
        self.assertEqual(foo.quux, 42)
        self.assertEqual(Foo.bar.__doc__, "<wrapped by the cachedproperty decorator>")
        self.assertEqual(Foo.quux.__doc__, "<wrapped by the cachedproperty decorator>\nsome prop")

        foo2 = Foo()
        self.assertEqual(foo2.bar, 2)
        # make sure foo.foo is cached
        self.assertEqual(foo.bar, 1)

        class Kallable:
            def __call__(self):
                return 42

        self.assertRaises(TypeError, cachedproperty, Kallable())


if __name__ == "__main__":
    unittest_main()



# copyright 2003-2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This file is part of logilab-common.
#
# logilab-common is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option) any
# later version.
#
# logilab-common is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with logilab-common.  If not, see <http://www.gnu.org/licenses/>.
"""
Unittests for table management
"""


import sys

from logilab.common.compat import StringIO
from logilab.common.testlib import TestCase, unittest_main
from logilab.common.table import (
    Table,
    TableStyleSheet,
    DocbookTableWriter,
    DocbookRenderer,
    TableStyle,
    TableWriter,
    TableCellRenderer,
)


class TableTC(TestCase):
    """Table TestCase class"""

    def setUp(self):
        """Creates a default table"""
        # from logilab.common import table
        # reload(table)
        self.table = Table()
        self.table.create_rows(["row1", "row2", "row3"])
        self.table.create_columns(["col1", "col2"])

    def test_valeur_scalaire(self):
        tab = Table()
        tab.create_columns(["col1"])
        tab.append_row([1])
        self.assertEqual(tab, [[1]])
        tab.append_row([2])
        self.assertEqual(tab[0, 0], 1)
        self.assertEqual(tab[1, 0], 2)

    def test_valeur_ligne(self):
        tab = Table()
        tab.create_columns(["col1", "col2"])
        tab.append_row([1, 2])
        self.assertEqual(tab, [[1, 2]])

    def test_valeur_colonne(self):
        tab = Table()
        tab.create_columns(["col1"])
        tab.append_row([1])
        tab.append_row([2])
        self.assertEqual(tab, [[1], [2]])
        self.assertEqual(tab[:, 0], [1, 2])

    def test_indexation(self):
        """we should be able to use [] to access rows"""
        self.assertEqual(self.table[0], self.table.data[0])
        self.assertEqual(self.table[1], self.table.data[1])

    def test_iterable(self):
        """test iter(table)"""
        it = iter(self.table)
        self.assertEqual(next(it), self.table.data[0])
        self.assertEqual(next(it), self.table.data[1])

    def test_get_rows(self):
        """tests Table.get_rows()"""
        self.assertEqual(self.table, [[0, 0], [0, 0], [0, 0]])
        self.assertEqual(self.table[:], [[0, 0], [0, 0], [0, 0]])
        self.table.insert_column(1, range(3), "supp")
        self.assertEqual(self.table, [[0, 0, 0], [0, 1, 0], [0, 2, 0]])
        self.assertEqual(self.table[:], [[0, 0, 0], [0, 1, 0], [0, 2, 0]])

    def test_get_cells(self):
        self.table.insert_column(1, range(3), "supp")
        self.assertEqual(self.table[0, 1], 0)
        self.assertEqual(self.table[1, 1], 1)
        self.assertEqual(self.table[2, 1], 2)
        self.assertEqual(self.table["row1", "supp"], 0)
        self.assertEqual(self.table["row2", "supp"], 1)
        self.assertEqual(self.table["row3", "supp"], 2)
        self.assertRaises(KeyError, self.table.__getitem__, ("row1", "foo"))
        self.assertRaises(KeyError, self.table.__getitem__, ("foo", "bar"))

    def test_shape(self):
        """tests table shape"""
        self.assertEqual(self.table.shape, (3, 2))
        self.table.insert_column(1, range(3), "supp")
        self.assertEqual(self.table.shape, (3, 3))

    def test_set_column(self):
        """Tests that table.set_column() works fine."""
        self.table.set_column(0, range(3))
        self.assertEqual(self.table[0, 0], 0)
        self.assertEqual(self.table[1, 0], 1)
        self.assertEqual(self.table[2, 0], 2)

    def test_set_column_by_id(self):
        """Tests that table.set_column_by_id() works fine."""
        self.table.set_column_by_id("col1", range(3))
        self.assertEqual(self.table[0, 0], 0)
        self.assertEqual(self.table[1, 0], 1)
        self.assertEqual(self.table[2, 0], 2)
        self.assertRaises(KeyError, self.table.set_column_by_id, "col123", range(3))

    def test_cells_ids(self):
        """tests that we can access cells by giving row/col ids"""
        self.assertRaises(KeyError, self.table.set_cell_by_ids, "row12", "col1", 12)
        self.assertRaises(KeyError, self.table.set_cell_by_ids, "row1", "col12", 12)
        self.assertEqual(self.table[0, 0], 0)
        self.table.set_cell_by_ids("row1", "col1", "DATA")
        self.assertEqual(self.table[0, 0], "DATA")
        self.assertRaises(KeyError, self.table.set_row_by_id, "row12", [])
        self.table.set_row_by_id("row1", ["1.0", "1.1"])
        self.assertEqual(self.table[0, 0], "1.0")

    def test_insert_row(self):
        """tests a row insertion"""
        tmp_data = ["tmp1", "tmp2"]
        self.table.insert_row(1, tmp_data, "tmprow")
        self.assertEqual(self.table[1], tmp_data)
        self.assertEqual(self.table["tmprow"], tmp_data)
        self.table.delete_row_by_id("tmprow")
        self.assertRaises(KeyError, self.table.delete_row_by_id, "tmprow")
        self.assertEqual(self.table[1], [0, 0])
        self.assertRaises(KeyError, self.table.__getitem__, "tmprow")

    def test_get_column(self):
        """Tests that table.get_column() works fine."""
        self.table.set_cell(0, 1, 12)
        self.table.set_cell(2, 1, 13)
        self.assertEqual(self.table[:, 1], [12, 0, 13])
        self.assertEqual(self.table[:, "col2"], [12, 0, 13])

    def test_get_columns(self):
        """Tests if table.get_columns() works fine."""
        self.table.set_cell(0, 1, 12)
        self.table.set_cell(2, 1, 13)
        self.assertEqual(self.table.get_columns(), [[0, 0, 0], [12, 0, 13]])

    def test_insert_column(self):
        """Tests that table.insert_column() works fine."""
        self.table.insert_column(1, range(3), "inserted_column")
        self.assertEqual(self.table[:, 1], [0, 1, 2])
        self.assertEqual(self.table.col_names, ["col1", "inserted_column", "col2"])

    def test_delete_column(self):
        """Tests that table.delete_column() works fine."""
        self.table.delete_column(1)
        self.assertEqual(self.table.col_names, ["col1"])
        self.assertEqual(self.table[:, 0], [0, 0, 0])
        self.assertRaises(KeyError, self.table.delete_column_by_id, "col2")
        self.table.delete_column_by_id("col1")
        self.assertEqual(self.table.col_names, [])

    def test_transpose(self):
        """Tests that table.transpose() works fine."""
        self.table.append_column(range(5, 8), "col3")
        ttable = self.table.transpose()
        self.assertEqual(ttable.row_names, ["col1", "col2", "col3"])
        self.assertEqual(ttable.col_names, ["row1", "row2", "row3"])
        self.assertEqual(ttable.data, [[0, 0, 0], [0, 0, 0], [5, 6, 7]])

    def test_sort_table(self):
        """Tests the table sort by column"""
        self.table.set_column(0, [3, 1, 2])
        self.table.set_column(1, [1, 2, 3])
        self.table.sort_by_column_index(0)
        self.assertEqual(self.table.row_names, ["row2", "row3", "row1"])
        self.assertEqual(self.table.data, [[1, 2], [2, 3], [3, 1]])
        self.table.sort_by_column_index(1, "desc")
        self.assertEqual(self.table.row_names, ["row3", "row2", "row1"])
        self.assertEqual(self.table.data, [[2, 3], [1, 2], [3, 1]])

    def test_sort_by_id(self):
        """tests sort_by_column_id()"""
        self.table.set_column_by_id("col1", [3, 1, 2])
        self.table.set_column_by_id("col2", [1, 2, 3])
        self.table.sort_by_column_id("col1")
        self.assertRaises(KeyError, self.table.sort_by_column_id, "col123")
        self.assertEqual(self.table.row_names, ["row2", "row3", "row1"])
        self.assertEqual(self.table.data, [[1, 2], [2, 3], [3, 1]])
        self.table.sort_by_column_id("col2", "desc")
        self.assertEqual(self.table.row_names, ["row3", "row2", "row1"])
        self.assertEqual(self.table.data, [[2, 3], [1, 2], [3, 1]])

    def test_pprint(self):
        """only tests pprint doesn't raise an exception"""
        self.table.pprint()
        str(self.table)


class GroupByTC(TestCase):
    """specific test suite for groupby()"""

    def setUp(self):
        t = Table()
        t.create_columns(["date", "res", "task", "usage"])
        t.append_row(["date1", "ing1", "task1", 0.3])
        t.append_row(["date1", "ing2", "task2", 0.3])
        t.append_row(["date2", "ing3", "task3", 0.3])
        t.append_row(["date3", "ing4", "task2", 0.3])
        t.append_row(["date1", "ing1", "task3", 0.3])
        t.append_row(["date3", "ing1", "task3", 0.3])
        self.table = t

    def test_single_groupby(self):
        """tests groupby() on several columns"""
        grouped = self.table.groupby("date")
        self.assertEqual(len(grouped), 3)
        self.assertEqual(len(grouped["date1"]), 3)
        self.assertEqual(len(grouped["date2"]), 1)
        self.assertEqual(len(grouped["date3"]), 2)
        self.assertEqual(
            grouped["date1"],
            [
                ("date1", "ing1", "task1", 0.3),
                ("date1", "ing2", "task2", 0.3),
                ("date1", "ing1", "task3", 0.3),
            ],
        )
        self.assertEqual(grouped["date2"], [("date2", "ing3", "task3", 0.3)])
        self.assertEqual(
            grouped["date3"],
            [
                ("date3", "ing4", "task2", 0.3),
                ("date3", "ing1", "task3", 0.3),
            ],
        )

    def test_multiple_groupby(self):
        """tests groupby() on several columns"""
        grouped = self.table.groupby("date", "task")
        self.assertEqual(len(grouped), 3)
        self.assertEqual(len(grouped["date1"]), 3)
        self.assertEqual(len(grouped["date2"]), 1)
        self.assertEqual(len(grouped["date3"]), 2)
        self.assertEqual(grouped["date1"]["task1"], [("date1", "ing1", "task1", 0.3)])
        self.assertEqual(grouped["date2"]["task3"], [("date2", "ing3", "task3", 0.3)])
        self.assertEqual(grouped["date3"]["task2"], [("date3", "ing4", "task2", 0.3)])
        date3 = grouped["date3"]
        self.assertRaises(KeyError, date3.__getitem__, "task1")

    def test_select(self):
        """tests Table.select() method"""
        rows = self.table.select("date", "date1")
        self.assertEqual(
            rows,
            [
                ("date1", "ing1", "task1", 0.3),
                ("date1", "ing2", "task2", 0.3),
                ("date1", "ing1", "task3", 0.3),
            ],
        )


class TableStyleSheetTC(TestCase):
    """The Stylesheet test case"""

    def setUp(self):
        """Builds a simple table to test the stylesheet"""
        self.table = Table()
        self.table.create_row("row1")
        self.table.create_columns(["a", "b", "c"])
        self.stylesheet = TableStyleSheet()
        # We don't want anything to be printed
        self.stdout_backup = sys.stdout
        sys.stdout = StringIO()

    def tearDown(self):
        sys.stdout = self.stdout_backup

    def test_add_rule(self):
        """Tests that the regex pattern works as expected."""
        rule = "0_2 = sqrt(0_0**2 + 0_1**2)"
        self.stylesheet.add_rule(rule)
        self.table.set_row(0, [3, 4, 0])
        self.table.apply_stylesheet(self.stylesheet)
        self.assertEqual(self.table[0], [3, 4, 5])
        self.assertEqual(len(self.stylesheet.rules), 1)
        self.stylesheet.add_rule("some bad rule with bad syntax")
        self.assertEqual(len(self.stylesheet.rules), 1, "Ill-formed rule mustn't be added")
        self.assertEqual(len(self.stylesheet.instructions), 1, "Ill-formed rule mustn't be added")

    def test_stylesheet_init(self):
        """tests Stylesheet.__init__"""
        rule = "0_2 = 1"
        sheet = TableStyleSheet([rule, "bad rule"])
        self.assertEqual(len(sheet.rules), 1, "Ill-formed rule mustn't be added")
        self.assertEqual(len(sheet.instructions), 1, "Ill-formed rule mustn't be added")

    def test_rowavg_rule(self):
        """Tests that add_rowavg_rule works as expected"""
        self.table.set_row(0, [10, 20, 0])
        self.stylesheet.add_rowavg_rule((0, 2), 0, 0, 1)
        self.table.apply_stylesheet(self.stylesheet)
        val = self.table[0, 2]
        self.assertEqual(int(val), 15)

    def test_rowsum_rule(self):
        """Tests that add_rowsum_rule works as expected"""
        self.table.set_row(0, [10, 20, 0])
        self.stylesheet.add_rowsum_rule((0, 2), 0, 0, 1)
        self.table.apply_stylesheet(self.stylesheet)
        val = self.table[0, 2]
        self.assertEqual(val, 30)

    def test_colavg_rule(self):
        """Tests that add_colavg_rule works as expected"""
        self.table.set_row(0, [10, 20, 0])
        self.table.append_row([12, 8, 3], "row2")
        self.table.create_row("row3")
        self.stylesheet.add_colavg_rule((2, 0), 0, 0, 1)
        self.table.apply_stylesheet(self.stylesheet)
        val = self.table[2, 0]
        self.assertEqual(int(val), 11)

    def test_colsum_rule(self):
        """Tests that add_colsum_rule works as expected"""
        self.table.set_row(0, [10, 20, 0])
        self.table.append_row([12, 8, 3], "row2")
        self.table.create_row("row3")
        self.stylesheet.add_colsum_rule((2, 0), 0, 0, 1)
        self.table.apply_stylesheet(self.stylesheet)
        val = self.table[2, 0]
        self.assertEqual(val, 22)


class TableStyleTC(TestCase):
    """Test suite for TableSuite"""

    def setUp(self):
        self.table = Table()
        self.table.create_rows(["row1", "row2", "row3"])
        self.table.create_columns(["col1", "col2"])
        self.style = TableStyle(self.table)
        self._tested_attrs = (("size", "1*"), ("alignment", "right"), ("unit", ""))

    def test_getset(self):
        """tests style's get and set methods"""
        for attrname, default_value in self._tested_attrs:
            getter = getattr(self.style, "get_%s" % attrname)
            setter = getattr(self.style, "set_%s" % attrname)
            self.assertRaises(KeyError, getter, "badcol")
            self.assertEqual(getter("col1"), default_value)
            setter("FOO", "col1")
            self.assertEqual(getter("col1"), "FOO")

    def test_getset_index(self):
        """tests style's get and set by index methods"""
        for attrname, default_value in self._tested_attrs:
            getter = getattr(self.style, "get_%s" % attrname)
            getattr(self.style, "set_%s" % attrname)
            igetter = getattr(self.style, "get_%s_by_index" % attrname)
            isetter = getattr(self.style, "set_%s_by_index" % attrname)
            self.assertEqual(getter("__row_column__"), default_value)
            isetter("FOO", 0)
            self.assertEqual(getter("__row_column__"), "FOO")
            self.assertEqual(igetter(0), "FOO")
            self.assertEqual(getter("col1"), default_value)
            isetter("FOO", 1)
            self.assertEqual(getter("col1"), "FOO")
            self.assertEqual(igetter(1), "FOO")


class RendererTC(TestCase):
    """Test suite for DocbookRenderer"""

    def setUp(self):
        self.renderer = DocbookRenderer(alignment=True)
        self.table = Table()
        self.table.create_rows(["row1", "row2", "row3"])
        self.table.create_columns(["col1", "col2"])
        self.style = TableStyle(self.table)
        self.base_renderer = TableCellRenderer()

    def test_cell_content(self):
        """test how alignment is rendered"""
        entry_xml = self.renderer._render_cell_content("data", self.style, 1)
        self.assertEqual(entry_xml, "<entry align='right'>data</entry>\n")
        self.style.set_alignment_by_index("left", 1)
        entry_xml = self.renderer._render_cell_content("data", self.style, 1)
        self.assertEqual(entry_xml, "<entry align='left'>data</entry>\n")

    def test_default_content_rendering(self):
        """tests that default rendering just prints the cell's content"""
        rendered_cell = self.base_renderer._render_cell_content("data", self.style, 1)
        self.assertEqual(rendered_cell, "data")

    def test_replacement_char(self):
        """tests that 0 is replaced when asked for"""
        cell_content = self.base_renderer._make_cell_content(0, self.style, 1)
        self.assertEqual(cell_content, 0)
        self.base_renderer.properties["skip_zero"] = "---"
        cell_content = self.base_renderer._make_cell_content(0, self.style, 1)
        self.assertEqual(cell_content, "---")

    def test_unit(self):
        """tests if units are added"""
        self.base_renderer.properties["units"] = True
        self.style.set_unit_by_index("EUR", 1)
        cell_content = self.base_renderer._make_cell_content(12, self.style, 1)
        self.assertEqual(cell_content, "12 EUR")


class DocbookTableWriterTC(TestCase):
    """TestCase for table's writer"""

    def setUp(self):
        self.stream = StringIO()
        self.table = Table()
        self.table.create_rows(["row1", "row2", "row3"])
        self.table.create_columns(["col1", "col2"])
        self.writer = DocbookTableWriter(self.stream, self.table, None)
        self.writer.set_renderer(DocbookRenderer())

    def test_write_table(self):
        """make sure write_table() doesn't raise any exception"""
        self.writer.write_table()

    def test_abstract_writer(self):
        """tests that Abstract Writers can't be used !"""
        writer = TableWriter(self.stream, self.table, None)
        self.assertRaises(NotImplementedError, writer.write_table)


if __name__ == "__main__":
    unittest_main()



# copyright 2003-2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This file is part of logilab-common.
#
# logilab-common is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option) any
# later version.
#
# logilab-common is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with logilab-common.  If not, see <http://www.gnu.org/licenses/>.
"""Python modules manipulation utility functions.

:type PY_SOURCE_EXTS: tuple(str)
:var PY_SOURCE_EXTS: list of possible python source file extension

:type STD_LIB_DIR: str
:var STD_LIB_DIR: directory where standard modules are located

:type BUILTIN_MODULES: dict
:var BUILTIN_MODULES: dictionary with builtin module names as key
"""

__docformat__ = "restructuredtext en"

import sys
import os
from os.path import (
    splitext,
    join,
    abspath,
    isdir,
    dirname,
    exists,
    expanduser,
    normcase,
    realpath,
)
from imp import find_module, load_module, C_BUILTIN, PY_COMPILED, PKG_DIRECTORY
from distutils.sysconfig import get_python_lib
from distutils.errors import DistutilsPlatformError
from typing import Dict, List, Optional, Any, Tuple, Union, Sequence
from types import ModuleType
from _frozen_importlib_external import FileFinder

from logilab.common import STD_BLACKLIST, _handle_blacklist
from logilab.common.deprecation import callable_deprecated

try:
    import zipimport
except ImportError:
    # mypy: Incompatible types in assignment (expression has type "None", variable has type Module)
    # conditional import
    zipimport = None  # type: ignore

ZIPFILE = object()

# Notes about STD_LIB_DIR
# Consider arch-specific installation for STD_LIB_DIR definition
# :mod:`distutils.sysconfig` contains to much hardcoded values to rely on
#
# :see: `Problems with /usr/lib64 builds <http://bugs.python.org/issue1294959>`_
# :see: `FHS <http://www.pathname.com/fhs/pub/fhs-2.3.html#LIBLTQUALGTALTERNATEFORMATESSENTIAL>`_
if sys.platform.startswith("win"):
    PY_SOURCE_EXTS = ("py", "pyw")
    PY_COMPILED_EXTS = ("dll", "pyd")
else:
    PY_SOURCE_EXTS = ("py",)
    PY_COMPILED_EXTS = ("so",)

try:
    STD_LIB_DIR = get_python_lib(standard_lib=True)
# get_python_lib(standard_lib=1) is not available on pypy, set STD_LIB_DIR to
# non-valid path, see https://bugs.pypy.org/issue1164
except DistutilsPlatformError:
    STD_LIB_DIR = "//"

EXT_LIB_DIR = get_python_lib()

BUILTIN_MODULES = dict.fromkeys(sys.builtin_module_names, True)


class NoSourceFile(Exception):
    """exception raised when we are not able to get a python
    source file for a precompiled file
    """


class LazyObject(object):
    """
    This class allows to lazyly declare a object (most likely only a callable
    according to the code) from a module without importing it.

    The import will be triggered when the user tries to access attributes of
    the object/callable or call it.

    Trying to set or delete attributes of the wrapped object/callable will not
    works as expected.
    """

    def __init__(self, module, obj):
        self.module = module
        self.obj = obj
        self._imported = None

    def _getobj(self):
        if self._imported is None:
            self._imported = getattr(load_module_from_name(self.module), self.obj)
        return self._imported

    def __getattribute__(self, attr):
        try:
            return super(LazyObject, self).__getattribute__(attr)
        except AttributeError:
            return getattr(self._getobj(), attr)

    def __call__(self, *args, **kwargs):
        return self._getobj()(*args, **kwargs)


def load_module_from_name(
    dotted_name: str, path: Optional[Any] = None, use_sys: int = True
) -> ModuleType:
    """Load a Python module from its name.

    :type dotted_name: str
    :param dotted_name: python name of a module or package

    :type path: list or None
    :param path:
      optional list of path where the module or package should be
      searched (use sys.path if nothing or None is given)

    :type use_sys: bool
    :param use_sys:
      boolean indicating whether the sys.modules dictionary should be
      used or not


    :raise ImportError: if the module or package is not found

    :rtype: module
    :return: the loaded module
    """
    module = load_module_from_modpath(dotted_name.split("."), path, use_sys)
    if module is None:
        raise ImportError("module %s doesn't exist" % dotted_name)
    return module


def load_module_from_modpath(
    parts: List[str], path: Optional[Any] = None, use_sys: int = True
) -> Optional[ModuleType]:
    """Load a python module from its splitted name.

    :type parts: list(str) or tuple(str)
    :param parts:
      python name of a module or package splitted on '.'

    :type path: list or None
    :param path:
      optional list of path where the module or package should be
      searched (use sys.path if nothing or None is given)

    :type use_sys: bool
    :param use_sys:
      boolean indicating whether the sys.modules dictionary should be used or not

    :raise ImportError: if the module or package is not found

    :rtype: module
    :return: the loaded module
    """
    if use_sys:
        try:
            return sys.modules[".".join(parts)]
        except KeyError:
            pass
    modpath = []
    prevmodule = None
    for part in parts:
        modpath.append(part)
        curname = ".".join(modpath)
        module = None
        if len(modpath) != len(parts):
            # even with use_sys=False, should try to get outer packages from sys.modules
            module = sys.modules.get(curname)
        elif use_sys:
            # because it may have been indirectly loaded through a parent
            module = sys.modules.get(curname)
        if module is None:
            mp_file, mp_filename, mp_desc = find_module(part, path)
            try:
                # mypy: Argument 2 to "load_module" has incompatible type "IO[Any]";
                # mypy: expected "Optional[_FileLike]"
                # this is handled by the try/finally somehow?
                module = load_module(curname, mp_file, mp_filename, mp_desc)  # type: ignore
            finally:
                if mp_file is not None:
                    mp_file.close()
        if prevmodule:
            setattr(prevmodule, part, module)
        _file = getattr(module, "__file__", "")
        prevmodule = module
        if not _file and _is_namespace(curname):
            continue
        if not _file and len(modpath) != len(parts):
            raise ImportError("no module in %s" % ".".join(parts[len(modpath) :]))
        path = [dirname(_file)]
    return module


def load_module_from_file(filepath, path=None, use_sys=True, extrapath=None):
    """Load a Python module from it's path.

    :type filepath: str
    :param filepath: path to the python module or package

    :type path: list or None
    :param path:
      optional list of path where the module or package should be
      searched (use sys.path if nothing or None is given)

    :type use_sys: bool
    :param use_sys:
      boolean indicating whether the sys.modules dictionary should be
      used or not


    :raise ImportError: if the module or package is not found

    :rtype: module
    :return: the loaded module
    """
    modpath = modpath_from_file(filepath, extrapath)
    return load_module_from_modpath(modpath, path, use_sys)


def _check_init(path: str, mod_path: List[str]) -> bool:
    """check there are some __init__.py all along the way"""
    modpath = []
    for part in mod_path:
        modpath.append(part)
        path = join(path, part)
        if not _is_namespace(".".join(modpath)) and not _has_init(path):
            return False
    return True


def _canonicalize_path(path: str) -> str:
    return realpath(expanduser(path))


@callable_deprecated("you should avoid using modpath_from_file()")
def modpath_from_file(filename: str, extrapath: Optional[Dict[str, str]] = None) -> List[str]:
    """DEPRECATED: doens't play well with symlinks and sys.meta_path

    Given a file path return the corresponding splitted module's name
    (i.e name of a module or package splitted on '.')

    :type filename: str
    :param filename: file's path for which we want the module's name

    :type extrapath: dict
    :param extrapath:
      optional extra search path, with path as key and package name for the path
      as value. This is usually useful to handle package splitted in multiple
      directories using __path__ trick.


    :raise ImportError:
      if the corresponding module's name has not been found

    :rtype: list(str)
    :return: the corresponding splitted module's name
    """
    filename = _canonicalize_path(filename)
    base = os.path.splitext(filename)[0]

    if extrapath is not None:
        for path_ in map(_canonicalize_path, extrapath):
            path = abspath(path_)
            if path and normcase(base[: len(path)]) == normcase(path):
                submodpath = [pkg for pkg in base[len(path) :].split(os.sep) if pkg]
                if _check_init(path, submodpath[:-1]):
                    return extrapath[path_].split(".") + submodpath

    for path in map(_canonicalize_path, sys.path):
        if path and normcase(base).startswith(path):
            modpath = [pkg for pkg in base[len(path) :].split(os.sep) if pkg]
            if _check_init(path, modpath[:-1]):
                return modpath

    raise ImportError("Unable to find module for %s in %s" % (filename, ", \n".join(sys.path)))


def file_from_modpath(
    modpath: List[str], path: Optional[Any] = None, context_file: Optional[str] = None
) -> Optional[str]:
    """given a mod path (i.e. splitted module / package name), return the
    corresponding file, giving priority to source file over precompiled
    file if it exists

    :type modpath: list or tuple
    :param modpath:
      splitted module's name (i.e name of a module or package splitted
      on '.')
      (this means explicit relative imports that start with dots have
      empty strings in this list!)

    :type path: list or None
    :param path:
      optional list of path where the module or package should be
      searched (use sys.path if nothing or None is given)

    :type context_file: str or None
    :param context_file:
      context file to consider, necessary if the identifier has been
      introduced using a relative import unresolvable in the actual
      context (i.e. modutils)

    :raise ImportError: if there is no such module in the directory

    :rtype: str or None
    :return:
      the path to the module's file or None if it's an integrated
      builtin module such as 'sys'
    """
    context: Optional[str]
    if context_file is not None:
        context = dirname(context_file)
    else:
        context = context_file
    if modpath[0] == "xml":
        # handle _xmlplus
        try:
            return _file_from_modpath(["_xmlplus"] + modpath[1:], path, context)
        except ImportError:
            return _file_from_modpath(modpath, path, context)
    elif modpath == ["os", "path"]:
        # FIXME: currently ignoring search_path...
        return os.path.__file__
    return _file_from_modpath(modpath, path, context)


def get_module_part(dotted_name: str, context_file: Optional[str] = None) -> str:
    """given a dotted name return the module part of the name :

    >>> get_module_part('logilab.common.modutils.get_module_part')
    'logilab.common.modutils'

    :type dotted_name: str
    :param dotted_name: full name of the identifier we are interested in

    :type context_file: str or None
    :param context_file:
      context file to consider, necessary if the identifier has been
      introduced using a relative import unresolvable in the actual
      context (i.e. modutils)


    :raise ImportError: if there is no such module in the directory

    :rtype: str or None
    :return:
      the module part of the name or None if we have not been able at
      all to import the given name

    XXX: deprecated, since it doesn't handle package precedence over module
    (see #10066)
    """
    # os.path trick
    if dotted_name.startswith("os.path"):
        return "os.path"
    parts = dotted_name.split(".")
    if context_file is not None:
        # first check for builtin module which won't be considered latter
        # in that case (path != None)
        if parts[0] in BUILTIN_MODULES:
            if len(parts) > 2:
                raise ImportError(dotted_name)
            return parts[0]
        # don't use += or insert, we want a new list to be created !
    path: Optional[List] = None
    starti = 0
    if parts[0] == "":
        assert context_file is not None, "explicit relative import, but no context_file?"
        path = []  # prevent resolving the import non-relatively
        starti = 1
    while parts[starti] == "":  # for all further dots: change context
        starti += 1
        assert context_file is not None
        context_file = dirname(context_file)
    for i in range(starti, len(parts)):
        try:
            file_from_modpath(parts[starti : i + 1], path=path, context_file=context_file)
        except ImportError:
            if not i >= max(1, len(parts) - 2):
                raise
            return ".".join(parts[:i])
    return dotted_name


def get_modules(
    package: str, src_directory: str, blacklist: Sequence[str] = STD_BLACKLIST
) -> List[str]:
    """given a package directory return a list of all available python
    modules in the package and its subpackages

    :type package: str
    :param package: the python name for the package

    :type src_directory: str
    :param src_directory:
      path of the directory corresponding to the package

    :type blacklist: list or tuple
    :param blacklist:
      optional list of files or directory to ignore, default to
      the value of `logilab.common.STD_BLACKLIST`

    :rtype: list
    :return:
      the list of all available python modules in the package and its
      subpackages
    """
    modules = []
    for directory, dirnames, filenames in os.walk(src_directory):
        _handle_blacklist(blacklist, dirnames, filenames)
        # check for __init__.py
        if "__init__.py" not in filenames:
            dirnames[:] = ()
            continue
        if directory != src_directory:
            dir_package = directory[len(src_directory) :].replace(os.sep, ".")
            modules.append(package + dir_package)
        for filename in filenames:
            if _is_python_file(filename) and filename != "__init__.py":
                src = join(directory, filename)
                module = package + src[len(src_directory) : -3]
                modules.append(module.replace(os.sep, "."))
    return modules


def get_module_files(src_directory: str, blacklist: Sequence[str] = STD_BLACKLIST) -> List[str]:
    """given a package directory return a list of all available python
    module's files in the package and its subpackages

    :type src_directory: str
    :param src_directory:
      path of the directory corresponding to the package

    :type blacklist: list or tuple
    :param blacklist:
      optional list of files or directory to ignore, default to the value of
      `logilab.common.STD_BLACKLIST`

    :rtype: list
    :return:
      the list of all available python module's files in the package and
      its subpackages
    """
    files = []
    for directory, dirnames, filenames in os.walk(src_directory):
        _handle_blacklist(blacklist, dirnames, filenames)
        # check for __init__.py
        if "__init__.py" not in filenames:
            dirnames[:] = ()
            continue
        for filename in filenames:
            if _is_python_file(filename):
                src = join(directory, filename)
                files.append(src)
    return files


def get_source_file(filename: str, include_no_ext: bool = False) -> str:
    """given a python module's file name return the matching source file
    name (the filename will be returned identically if it's a already an
    absolute path to a python source file...)

    :type filename: str
    :param filename: python module's file name


    :raise NoSourceFile: if no source file exists on the file system

    :rtype: str
    :return: the absolute path of the source file if it exists
    """
    base, orig_ext = splitext(abspath(filename))
    for ext in PY_SOURCE_EXTS:
        source_path = "%s.%s" % (base, ext)
        if exists(source_path):
            return source_path
    if include_no_ext and not orig_ext and exists(base):
        return base
    raise NoSourceFile(filename)


def cleanup_sys_modules(directories):
    """remove submodules of `directories` from `sys.modules`"""
    cleaned = []
    for modname, module in list(sys.modules.items()):
        modfile = getattr(module, "__file__", None)
        if modfile:
            for directory in directories:
                if modfile.startswith(directory):
                    cleaned.append(modname)
                    del sys.modules[modname]
                    break
    return cleaned


def clean_sys_modules(names):
    """remove submodules starting with name from `names` from `sys.modules`"""
    cleaned = set()
    for modname in list(sys.modules):
        for name in names:
            if modname.startswith(name):
                del sys.modules[modname]
                cleaned.add(modname)
                break
    return cleaned


def is_python_source(filename):
    """
    rtype: bool
    return: True if the filename is a python source file
    """
    return splitext(filename)[1][1:] in PY_SOURCE_EXTS


def is_standard_module(
    modname: str, std_path: Union[List[str], Tuple[str]] = (STD_LIB_DIR,)
) -> bool:
    """try to guess if a module is a standard python module (by default,
    see `std_path` parameter's description)

    :type modname: str
    :param modname: name of the module we are interested in

    :type std_path: list(str) or tuple(str)
    :param std_path: list of path considered as standard


    :rtype: bool
    :return:
      true if the module:
      - is located on the path listed in one of the directory in `std_path`
      - is a built-in module

    Note: this function is known to return wrong values when inside virtualenv.
    See https://www.logilab.org/ticket/294756.
    """
    modname = modname.split(".")[0]
    try:
        filename = file_from_modpath([modname])
    except ImportError:
        # import failed, i'm probably not so wrong by supposing it's
        # not standard...
        return False
    # modules which are not living in a file are considered standard
    # (sys and __builtin__ for instance)
    if filename is None:
        # we assume there are no namespaces in stdlib
        return not _is_namespace(modname)
    filename = abspath(filename)
    if filename.startswith(EXT_LIB_DIR):
        return False
    for path in std_path:
        if filename.startswith(abspath(path)):
            return True
    return False


def is_relative(modname: str, from_file: str) -> bool:
    """return true if the given module name is relative to the given
    file name

    :type modname: str
    :param modname: name of the module we are interested in

    :type from_file: str
    :param from_file:
      path of the module from which modname has been imported

    :rtype: bool
    :return:
      true if the module has been imported relatively to `from_file`
    """
    if not isdir(from_file):
        from_file = dirname(from_file)
    if from_file in sys.path:
        return False
    try:
        find_module(modname.split(".")[0], [from_file])
        return True
    except ImportError:
        return False


# internal only functions #####################################################


def _file_from_modpath(
    modpath: List[str], path: Optional[Any] = None, context: Optional[str] = None
) -> Optional[str]:
    """given a mod path (i.e. splitted module / package name), return the
    corresponding file

    this function is used internally, see `file_from_modpath`'s
    documentation for more information
    """
    assert len(modpath) > 0
    if context is not None:
        try:
            mtype, mp_filename = _module_file(modpath, [context])
        except ImportError:
            mtype, mp_filename = _module_file(modpath, path)
    else:
        mtype, mp_filename = _module_file(modpath, path)
    if mtype == PY_COMPILED:
        try:
            assert mp_filename is not None
            return get_source_file(mp_filename)
        except NoSourceFile:
            return mp_filename
    elif mtype == C_BUILTIN:
        # integrated builtin module
        return None
    elif mtype == PKG_DIRECTORY:
        assert mp_filename is not None
        mp_filename = _has_init(mp_filename)
    return mp_filename


def _search_zip(
    modpath: List[str], pic: Dict[str, Optional[FileFinder]]
) -> Tuple[object, str, str]:
    for filepath, importer in pic.items():
        if importer is not None:
            if importer.find_module(modpath[0]):
                if not importer.find_module("/".join(modpath)):
                    raise ImportError(
                        "No module named %s in %s/%s" % (".".join(modpath[1:]), filepath, modpath)
                    )
                return ZIPFILE, abspath(filepath) + "/" + "/".join(modpath), filepath
    raise ImportError("No module named %s" % ".".join(modpath))


try:
    import pkg_resources
except ImportError:
    # mypy: Incompatible types in assignment (expression has type "None", variable has type Module)
    # conditional import
    pkg_resources = None  # type: ignore


def _is_namespace(modname: str) -> bool:
    # mypy: Module has no attribute "_namespace_packages"; maybe "fixup_namespace_packages"?"
    # but is still has? or is it a failure from python3 port?
    return (
        pkg_resources is not None and modname in pkg_resources._namespace_packages  # type: ignore
    )


def _module_file(
    modpath: List[str], path: Optional[List[str]] = None
) -> Tuple[Union[int, object], Optional[str]]:
    """get a module type / file path

    :type modpath: list or tuple
    :param modpath:
      splitted module's name (i.e name of a module or package splitted
      on '.'), with leading empty strings for explicit relative import

    :type path: list or None
    :param path:
      optional list of path where the module or package should be
      searched (use sys.path if nothing or None is given)


    :rtype: tuple(int, str)
    :return: the module type flag and the file path for a module
    """
    # egg support compat
    try:
        pic = sys.path_importer_cache
        _path = path if path is not None else sys.path
        for __path in _path:
            if __path not in pic:
                try:
                    pic[__path] = zipimport.zipimporter(__path)
                except zipimport.ZipImportError:
                    pic[__path] = None
        checkeggs = True
    except AttributeError:
        checkeggs = False
    # pkg_resources support (aka setuptools namespace packages)
    if _is_namespace(modpath[0]) and modpath[0] in sys.modules:
        # setuptools has added into sys.modules a module object with proper
        # __path__, get back information from there
        module = sys.modules[modpath.pop(0)]
        # use list() to protect against _NamespacePath instance we get with python 3, which
        # find_module later doesn't like
        # mypy: Module has no attribute "__path__"
        # I guess it does thanks to logilab's magic?
        path = list(module.__path__)  # type: ignore
        if not modpath:
            return C_BUILTIN, None

    imported = []

    while modpath:
        modname = modpath[0]
        # take care to changes in find_module implementation wrt builtin modules
        #
        # Python 2.6.6 (r266:84292, Sep 11 2012, 08:34:23)
        # >>> imp.find_module('posix')
        # (None, 'posix', ('', '', 6))
        #
        # Python 3.3.1 (default, Apr 26 2013, 12:08:46)
        # >>> imp.find_module('posix')
        # (None, None, ('', '', 6))
        try:
            _, mp_filename, mp_desc = find_module(modname, path)
        except ImportError:
            if checkeggs:
                return _search_zip(modpath, pic)[:2]
            raise
        else:
            if checkeggs and mp_filename:
                fullabspath = [abspath(x) for x in _path]
                try:
                    pathindex = fullabspath.index(dirname(abspath(mp_filename)))
                    emtype, emp_filename, zippath = _search_zip(modpath, pic)
                    if pathindex > _path.index(zippath):
                        # an egg takes priority
                        return emtype, emp_filename
                except ValueError:
                    # XXX not in _path
                    pass
                except ImportError:
                    pass
                checkeggs = False
        imported.append(modpath.pop(0))
        mtype = mp_desc[2]
        if modpath:
            if mtype != PKG_DIRECTORY:
                raise ImportError("No module %s in %s" % (".".join(modpath), ".".join(imported)))
            # XXX guess if package is using pkgutil.extend_path by looking for
            # those keywords in the first four Kbytes
            try:
                with open(join(mp_filename, "__init__.py")) as stream:
                    data = stream.read(4096)
            except IOError:
                path = [mp_filename]
            else:
                if "pkgutil" in data and "extend_path" in data:
                    # extend_path is called, search sys.path for module/packages
                    # of this name see pkgutil.extend_path documentation
                    path = [join(p, *imported) for p in sys.path if isdir(join(p, *imported))]
                else:
                    path = [mp_filename]
    return mtype, mp_filename


def _is_python_file(filename: str) -> bool:
    """return true if the given filename should be considered as a python file

    .pyc and .pyo are ignored
    """
    for ext in (".py", ".so", ".pyd", ".pyw"):
        if filename.endswith(ext):
            return True
    return False


def _has_init(directory: str) -> Optional[str]:
    """if the given directory has a valid __init__ file, return its path,
    else return None
    """
    mod_or_pack = join(directory, "__init__")

    for ext in PY_SOURCE_EXTS + ("pyc", "pyo"):
        if exists(mod_or_pack + "." + ext):
            return mod_or_pack + "." + ext

    return None



# copyright 2003-2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This file is part of logilab-common.
#
# logilab-common is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option) any
# later version.
#
# logilab-common is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with logilab-common.  If not, see <http://www.gnu.org/licenses/>.
"""unit tests for logilab.common.shellutils"""

from os.path import join, dirname, abspath
from unittest.mock import patch

from logilab.common.testlib import TestCase, unittest_main

from logilab.common.shellutils import globfind, find, ProgressBar, RawInput
from logilab.common.compat import StringIO


DATA_DIR = join(dirname(abspath(__file__)), "data", "find_test")


class FindTC(TestCase):
    def test_include(self):
        files = set(find(DATA_DIR, ".py"))
        self.assertSetEqual(
            files,
            {
                join(DATA_DIR, f)
                for f in [
                    "__init__.py",
                    "module.py",
                    "module2.py",
                    "noendingnewline.py",
                    "nonregr.py",
                    join("sub", "momo.py"),
                ]
            },
        )
        files = set(find(DATA_DIR, (".py",), blacklist=("sub",)))
        self.assertSetEqual(
            files,
            {
                join(DATA_DIR, f)
                for f in [
                    "__init__.py",
                    "module.py",
                    "module2.py",
                    "noendingnewline.py",
                    "nonregr.py",
                ]
            },
        )

    def test_exclude(self):
        files = set(find(DATA_DIR, (".py", ".pyc"), exclude=True))
        self.assertSetEqual(
            files,
            {
                join(DATA_DIR, f)
                for f in [
                    "foo.txt",
                    "newlines.txt",
                    "normal_file.txt",
                    "test.ini",
                    "test1.msg",
                    "test2.msg",
                    "spam.txt",
                    join("sub", "doc.txt"),
                    "write_protected_file.txt",
                ]
            },
        )

    def test_globfind(self):
        files = set(globfind(DATA_DIR, "*.py"))
        self.assertSetEqual(
            files,
            {
                join(DATA_DIR, f)
                for f in [
                    "__init__.py",
                    "module.py",
                    "module2.py",
                    "noendingnewline.py",
                    "nonregr.py",
                    join("sub", "momo.py"),
                ]
            },
        )
        files = set(globfind(DATA_DIR, "mo*.py"))
        self.assertSetEqual(
            files,
            {join(DATA_DIR, f) for f in ["module.py", "module2.py", join("sub", "momo.py")]},
        )
        files = set(globfind(DATA_DIR, "mo*.py", blacklist=("sub",)))
        self.assertSetEqual(files, {join(DATA_DIR, f) for f in ["module.py", "module2.py"]})


class ProgressBarTC(TestCase):
    def test_refresh(self):
        pgb_stream = StringIO()
        expected_stream = StringIO()
        pgb = ProgressBar(20, stream=pgb_stream)
        self.assertEqual(
            pgb_stream.getvalue(), expected_stream.getvalue()
        )  # nothing print before refresh
        pgb.refresh()
        expected_stream.write("\r[" + " " * 20 + "]")
        self.assertEqual(pgb_stream.getvalue(), expected_stream.getvalue())

    def test_refresh_g_size(self):
        pgb_stream = StringIO()
        expected_stream = StringIO()
        pgb = ProgressBar(20, 35, stream=pgb_stream)
        pgb.refresh()
        expected_stream.write("\r[" + " " * 35 + "]")
        self.assertEqual(pgb_stream.getvalue(), expected_stream.getvalue())

    def test_refresh_l_size(self):
        pgb_stream = StringIO()
        expected_stream = StringIO()
        pgb = ProgressBar(20, 3, stream=pgb_stream)
        pgb.refresh()
        expected_stream.write("\r[" + " " * 3 + "]")
        self.assertEqual(pgb_stream.getvalue(), expected_stream.getvalue())

    def _update_test(self, nbops, expected, size=None):
        pgb_stream = StringIO()
        expected_stream = StringIO()
        if size is None:
            pgb = ProgressBar(nbops, stream=pgb_stream)
            size = 20
        else:
            pgb = ProgressBar(nbops, size, stream=pgb_stream)
        last = 0
        for round in expected:
            if not hasattr(round, "__int__"):
                dots, update = round
            else:
                dots, update = round, None
            pgb.update()
            if update or (update is None and dots != last):
                last = dots
                expected_stream.write("\r[" + ("=" * dots) + (" " * (size - dots)) + "]")
            self.assertEqual(pgb_stream.getvalue(), expected_stream.getvalue())

    def test_default(self):
        self._update_test(20, range(1, 21))

    def test_nbops_gt_size(self):
        """Test the progress bar for nbops > size"""

        def half(total):
            for counter in range(1, total + 1):
                yield counter // 2

        self._update_test(40, half(40))

    def test_nbops_lt_size(self):
        """Test the progress bar for nbops < size"""

        def double(total):
            for counter in range(1, total + 1):
                yield counter * 2

        self._update_test(10, double(10))

    def test_nbops_nomul_size(self):
        """Test the progress bar for size % nbops !=0 (non int number of dots per update)"""
        self._update_test(3, (6, 13, 20))

    def test_overflow(self):
        self._update_test(5, (8, 16, 25, 33, 42, (42, True)), size=42)

    def test_update_exact(self):
        pgb_stream = StringIO()
        expected_stream = StringIO()
        size = 20
        pgb = ProgressBar(100, size, stream=pgb_stream)
        for dots in range(10, 105, 15):
            pgb.update(dots, exact=True)
            dots //= 5
            expected_stream.write("\r[" + ("=" * dots) + (" " * (size - dots)) + "]")
            self.assertEqual(pgb_stream.getvalue(), expected_stream.getvalue())

    def test_update_relative(self):
        pgb_stream = StringIO()
        expected_stream = StringIO()
        size = 20
        pgb = ProgressBar(100, size, stream=pgb_stream)
        for dots in range(5, 105, 5):
            pgb.update(5, exact=False)
            dots //= 5
            expected_stream.write("\r[" + ("=" * dots) + (" " * (size - dots)) + "]")
            self.assertEqual(pgb_stream.getvalue(), expected_stream.getvalue())


class RawInputTC(TestCase):
    def auto_input(self, *args):
        self.input_args = args
        return self.input_answer

    def setUp(self):
        null_printer = lambda x: None
        self.qa = RawInput(self.auto_input, null_printer)

    def test_ask_using_builtin_input(self):
        with patch("builtins.input", return_value="no"):
            qa = RawInput()
            answer = qa.ask("text", ("yes", "no"), "yes")
        self.assertEqual(answer, "no")

    def test_ask_default(self):
        self.input_answer = ""
        answer = self.qa.ask("text", ("yes", "no"), "yes")
        self.assertEqual(answer, "yes")
        self.input_answer = "  "
        answer = self.qa.ask("text", ("yes", "no"), "yes")
        self.assertEqual(answer, "yes")

    def test_ask_case(self):
        self.input_answer = "no"
        answer = self.qa.ask("text", ("yes", "no"), "yes")
        self.assertEqual(answer, "no")
        self.input_answer = "No"
        answer = self.qa.ask("text", ("yes", "no"), "yes")
        self.assertEqual(answer, "no")
        self.input_answer = "NO"
        answer = self.qa.ask("text", ("yes", "no"), "yes")
        self.assertEqual(answer, "no")
        self.input_answer = "nO"
        answer = self.qa.ask("text", ("yes", "no"), "yes")
        self.assertEqual(answer, "no")
        self.input_answer = "YES"
        answer = self.qa.ask("text", ("yes", "no"), "yes")
        self.assertEqual(answer, "yes")

    def test_ask_prompt(self):
        self.input_answer = ""
        self.qa.ask("text", ("yes", "no"), "yes")
        self.assertEqual(self.input_args[0], "text [Y(es)/n(o)]: ")
        self.qa.ask("text", ("y", "n"), "y")
        self.assertEqual(self.input_args[0], "text [Y/n]: ")
        self.qa.ask("text", ("n", "y"), "y")
        self.assertEqual(self.input_args[0], "text [n/Y]: ")
        self.qa.ask("text", ("yes", "no", "maybe", "1"), "yes")
        self.assertEqual(self.input_args[0], "text [Y(es)/n(o)/m(aybe)/1]: ")

    def test_ask_ambiguous(self):
        self.input_answer = "y"
        self.assertRaises(Exception, self.qa.ask, "text", ("yes", "yep"), "yes")

    def test_confirm(self):
        self.input_answer = "y"
        self.assertEqual(self.qa.confirm("Say yes"), True)
        self.assertEqual(self.qa.confirm("Say yes", default_is_yes=False), True)
        self.input_answer = "n"
        self.assertEqual(self.qa.confirm("Say yes"), False)
        self.assertEqual(self.qa.confirm("Say yes", default_is_yes=False), False)
        self.input_answer = ""
        self.assertEqual(self.qa.confirm("Say default"), True)
        self.assertEqual(self.qa.confirm("Say default", default_is_yes=False), False)


if __name__ == "__main__":
    unittest_main()



# copyright 2003-2014 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This file is part of logilab-common.
#
# logilab-common is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option) any
# later version.
#
# logilab-common is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with logilab-common.  If not, see <http://www.gnu.org/licenses/>.
import tempfile
import os
from os.path import join, dirname, abspath
import re

from sys import version_info

from logilab.common import attrdict
from logilab.common.compat import StringIO
from logilab.common.testlib import TestCase, unittest_main
from logilab.common.optik_ext import OptionValueError
from logilab.common.configuration import (
    Configuration,
    OptionError,
    OptionsManagerMixIn,
    OptionsProviderMixIn,
    Method,
    read_old_config,
    merge_options,
)

DATA = join(dirname(abspath(__file__)), "data")

OPTIONS = [
    ("dothis", {"type": "yn", "action": "store", "default": True, "metavar": "<y or n>"}),
    ("value", {"type": "string", "metavar": "<string>", "short": "v"}),
    (
        "multiple",
        {
            "type": "csv",
            "default": ["yop", "yep"],
            "metavar": "<comma separated values>",
            "help": "you can also document the option",
        },
    ),
    ("number", {"type": "int", "default": 2, "metavar": "<int>", "help": "boom"}),
    ("bytes", {"type": "bytes", "default": "1KB", "metavar": "<bytes>"}),
    ("choice", {"type": "choice", "default": "yo", "choices": ("yo", "ye"), "metavar": "<yo|ye>"}),
    (
        "multiple-choice",
        {
            "type": "multiple_choice",
            "default": ["yo", "ye"],
            "choices": ("yo", "ye", "yu", "yi", "ya"),
            "metavar": "<yo|ye>",
        },
    ),
    ("named", {"type": "named", "default": Method("get_named"), "metavar": "<key=val>"}),
    (
        "diffgroup",
        {"type": "string", "default": "pouet", "metavar": "<key=val>", "group": "agroup"},
    ),
    ("reset-value", {"type": "string", "metavar": "<string>", "short": "r", "dest": "value"}),
    ("opt-b-1", {"type": "string", "metavar": "<string>", "group": "bgroup"}),
    ("opt-b-2", {"type": "string", "metavar": "<string>", "group": "bgroup"}),
]


class MyConfiguration(Configuration):
    """test configuration"""

    def get_named(self):
        return {"key": "val"}


class ConfigurationTC(TestCase):
    def setUp(self):
        self.cfg = MyConfiguration(name="test", options=OPTIONS, usage="Just do it ! (tm)")

    def test_default(self):
        cfg = self.cfg
        self.assertEqual(cfg["dothis"], True)
        self.assertEqual(cfg["value"], None)
        self.assertEqual(cfg["multiple"], ["yop", "yep"])
        self.assertEqual(cfg["number"], 2)
        self.assertEqual(cfg["bytes"], 1024)
        self.assertIsInstance(cfg["bytes"], int)
        self.assertEqual(cfg["choice"], "yo")
        self.assertEqual(cfg["multiple-choice"], ["yo", "ye"])
        self.assertEqual(cfg["named"], {"key": "val"})

    def test_base(self):
        cfg = self.cfg
        cfg.set_option("number", "0")
        self.assertEqual(cfg["number"], 0)
        self.assertRaises(OptionValueError, cfg.set_option, "number", "youpi")
        self.assertRaises(OptionValueError, cfg.set_option, "choice", "youpi")
        self.assertRaises(OptionValueError, cfg.set_option, "multiple-choice", ("yo", "y", "ya"))
        cfg.set_option("multiple-choice", "yo, ya")
        self.assertEqual(cfg["multiple-choice"], ["yo", "ya"])
        self.assertEqual(cfg.get("multiple-choice"), ["yo", "ya"])
        self.assertEqual(cfg.get("whatever"), None)

    def test_load_command_line_configuration(self):
        cfg = self.cfg
        args = cfg.load_command_line_configuration(
            [
                "--choice",
                "ye",
                "--number",
                "4",
                "--multiple=1,2,3",
                "--dothis=n",
                "--bytes=10KB",
                "other",
                "arguments",
            ]
        )
        self.assertEqual(args, ["other", "arguments"])
        self.assertEqual(cfg["dothis"], False)
        self.assertEqual(cfg["multiple"], ["1", "2", "3"])
        self.assertEqual(cfg["number"], 4)
        self.assertEqual(cfg["bytes"], 10240)
        self.assertEqual(cfg["choice"], "ye")
        self.assertEqual(cfg["value"], None)
        args = cfg.load_command_line_configuration(["-v", "duh"])
        self.assertEqual(args, [])
        self.assertEqual(cfg["value"], "duh")
        self.assertEqual(cfg["dothis"], False)
        self.assertEqual(cfg["multiple"], ["1", "2", "3"])
        self.assertEqual(cfg["number"], 4)
        self.assertEqual(cfg["bytes"], 10240)
        self.assertEqual(cfg["choice"], "ye")

    def test_load_configuration(self):
        cfg = self.cfg
        cfg.load_configuration(
            choice="ye", number="4", multiple="1,2,3", dothis="n", multiple_choice=("yo", "ya")
        )
        self.assertEqual(cfg["dothis"], False)
        self.assertEqual(cfg["multiple"], ["1", "2", "3"])
        self.assertEqual(cfg["number"], 4)
        self.assertEqual(cfg["choice"], "ye")
        self.assertEqual(cfg["value"], None)
        self.assertEqual(cfg["multiple-choice"], ("yo", "ya"))

    def test_load_configuration_file_case_insensitive(self):
        file = tempfile.mktemp()
        stream = open(file, "w")
        try:
            stream.write(
                """[Test]

dothis=no

#value=

# you can also document the option
multiple=yop,yepii

# boom
number=3

bytes=1KB

choice=yo

multiple-choice=yo,ye

named=key:val


[agroup]

diffgroup=zou
"""
            )
            stream.close()
            self.cfg.load_file_configuration(file)
            self.assertEqual(self.cfg["dothis"], False)
            self.assertEqual(self.cfg["value"], None)
            self.assertEqual(self.cfg["multiple"], ["yop", "yepii"])
            self.assertEqual(self.cfg["diffgroup"], "zou")
        finally:
            os.remove(file)

    def test_option_order(self):
        """Check that options are taken into account in the command line order
        and not in the order they are defined in the Configuration object.
        """
        file = tempfile.mktemp()
        stream = open(file, "w")
        try:
            stream.write(
                """[Test]
reset-value=toto
value=tata
"""
            )
            stream.close()
            self.cfg.load_file_configuration(file)
        finally:
            os.remove(file)
        self.assertEqual(self.cfg["value"], "tata")

    def test_unsupported_options(self):
        file = tempfile.mktemp()
        stream = open(file, "w")
        try:
            stream.write(
                """[Test]
whatever=toto
value=tata
"""
            )
            stream.close()
            self.cfg.load_file_configuration(file)
        finally:
            os.remove(file)
        self.assertEqual(self.cfg["value"], "tata")
        self.assertRaises(OptionError, self.cfg.__getitem__, "whatever")

    def test_generate_config(self):
        stream = StringIO()
        self.cfg.generate_config(stream)
        self.assertMultiLineEqual(
            stream.getvalue().strip(),
            """[TEST]

dothis=yes

#value=

# you can also document the option
multiple=yop,yep

# boom
number=2

bytes=1KB

choice=yo

multiple-choice=yo,ye

named=key:val

#reset-value=


[AGROUP]

diffgroup=pouet


[BGROUP]

#opt-b-1=

#opt-b-2=""",
        )

    def test_generate_config_header(self):
        header_message = """This a multiline header
    All lines should be commented.
    Everything should be at the top of the file."""

        commented_header_message = """# This a multiline header
# All lines should be commented.
# Everything should be at the top of the file."""
        exepected_stream = StringIO()
        print(commented_header_message, file=exepected_stream)
        self.cfg.generate_config(exepected_stream)

        stream = StringIO()
        self.cfg.generate_config(stream, header_message=header_message)
        self.assertMultiLineEqual(stream.getvalue().strip(), exepected_stream.getvalue().strip())

    def test_generate_config_with_space_string(self):
        self.cfg["value"] = "    "
        stream = StringIO()
        self.cfg.generate_config(stream)
        self.assertMultiLineEqual(
            stream.getvalue().strip(),
            """[TEST]

dothis=yes

value='    '

# you can also document the option
multiple=yop,yep

# boom
number=2

bytes=1KB

choice=yo

multiple-choice=yo,ye

named=key:val

reset-value='    '


[AGROUP]

diffgroup=pouet


[BGROUP]

#opt-b-1=

#opt-b-2=""",
        )

    def test_generate_config_with_multiline_string(self):
        self.cfg["value"] = "line1\nline2\nline3"
        stream = StringIO()
        self.cfg.generate_config(stream)
        self.assertMultiLineEqual(
            stream.getvalue().strip(),
            """[TEST]

dothis=yes

value=
    line1
    line2
    line3

# you can also document the option
multiple=yop,yep

# boom
number=2

bytes=1KB

choice=yo

multiple-choice=yo,ye

named=key:val

reset-value=
    line1
    line2
    line3


[AGROUP]

diffgroup=pouet


[BGROUP]

#opt-b-1=

#opt-b-2=""",
        )

    def test_roundtrip(self):
        cfg = self.cfg
        f = tempfile.mktemp()
        stream = open(f, "w")
        try:
            self.cfg["dothis"] = False
            self.cfg["multiple"] = ["toto", "tata"]
            self.cfg["number"] = 3
            self.cfg["bytes"] = 2048
            cfg.generate_config(stream)
            stream.close()
            new_cfg = MyConfiguration(name="test", options=OPTIONS)
            new_cfg.load_file_configuration(f)
            self.assertEqual(cfg["dothis"], new_cfg["dothis"])
            self.assertEqual(cfg["multiple"], new_cfg["multiple"])
            self.assertEqual(cfg["number"], new_cfg["number"])
            self.assertEqual(cfg["bytes"], new_cfg["bytes"])
            self.assertEqual(cfg["choice"], new_cfg["choice"])
            self.assertEqual(cfg["value"], new_cfg["value"])
            self.assertEqual(cfg["multiple-choice"], new_cfg["multiple-choice"])
        finally:
            os.remove(f)

    def test_setitem(self):
        self.assertRaises(OptionValueError, self.cfg.__setitem__, "multiple-choice", ("a", "b"))
        self.cfg["multiple-choice"] = ("yi", "ya")
        self.assertEqual(self.cfg["multiple-choice"], ("yi", "ya"))

    def test_help(self):
        self.cfg.add_help_section("bonus", "a nice additional help")
        help = self.cfg.help().strip()
        # at least in python 2.4.2 the output is:
        # '  -v <string>, --value=<string>'
        # it is not unlikely some optik/optparse versions do print -v<string>
        # so accept both
        help = help.replace(" -v <string>, ", " -v<string>, ")
        help = re.sub("[ ]*(\r?\n)", "\\1", help)
        USAGE = """Usage: Just do it ! (tm)

Options:
  -h, --help            show this help message and exit
  --dothis=<y or n>
  -v<string>, --value=<string>
  --multiple=<comma separated values>
                        you can also document the option [current: yop,yep]
  --number=<int>        boom [current: 2]
  --bytes=<bytes>
  --choice=<yo|ye>
  --multiple-choice=<yo|ye>
  --named=<key=val>
  -r <string>, --reset-value=<string>

  Agroup:
    --diffgroup=<key=val>

  Bgroup:
    --opt-b-1=<string>
    --opt-b-2=<string>

  Bonus:
    a nice additional help"""
        if version_info < (2, 5):
            # 'usage' header is not capitalized in this version
            USAGE = USAGE.replace("Usage: ", "usage: ")
        elif version_info < (2, 4):
            USAGE = """usage: Just do it ! (tm)

options:
  -h, --help            show this help message and exit
  --dothis=<y or n>
  -v<string>, --value=<string>
  --multiple=<comma separated values>
                        you can also document the option
  --number=<int>
  --choice=<yo|ye>
  --multiple-choice=<yo|ye>
  --named=<key=val>

  Bonus:
    a nice additional help
"""
        self.assertMultiLineEqual(help, USAGE)

    def test_manpage(self):
        pkginfo = {}
        with open(join(DATA, "__pkginfo__.py")) as fobj:
            exec(fobj.read(), pkginfo)
        self.cfg.generate_manpage(attrdict(pkginfo), stream=StringIO())

    def test_rewrite_config(self):
        changes = [
            ("renamed", "renamed", "choice"),
            ("moved", "named", "old", "test"),
        ]
        read_old_config(self.cfg, changes, join(DATA, "test.ini"))
        stream = StringIO()
        self.cfg.generate_config(stream)
        self.assertMultiLineEqual(
            stream.getvalue().strip(),
            """[TEST]

dothis=yes

value='    '

# you can also document the option
multiple=yop

# boom
number=2

bytes=1KB

choice=yo

multiple-choice=yo,ye

named=key:val

reset-value='    '


[AGROUP]

diffgroup=pouet


[BGROUP]

#opt-b-1=

#opt-b-2=""",
        )


class Linter(OptionsManagerMixIn, OptionsProviderMixIn):
    options = (
        (
            "profile",
            {"type": "yn", "metavar": "<y_or_n>", "default": False, "help": "Profiled execution."},
        ),
    )

    def __init__(self):
        OptionsManagerMixIn.__init__(self, usage="")
        OptionsProviderMixIn.__init__(self)
        self.register_options_provider(self)
        self.load_provider_defaults()


class RegrTC(TestCase):
    def setUp(self):
        self.linter = Linter()

    def test_load_defaults(self):
        self.linter.load_command_line_configuration([])
        self.assertEqual(self.linter.config.profile, False)

    def test_register_options_multiple_groups(self):
        """ensure multiple option groups can be registered at once"""
        config = Configuration()
        self.assertEqual(config.options, ())
        new_options = (
            ("option1", {"type": "string", "help": "", "group": "g1", "level": 2}),
            ("option2", {"type": "string", "help": "", "group": "g1", "level": 2}),
            ("option3", {"type": "string", "help": "", "group": "g2", "level": 2}),
        )
        config.register_options(new_options)
        self.assertEqual(config.options, new_options)


class MergeTC(TestCase):
    def test_merge1(self):
        merged = merge_options(
            [
                (
                    "dothis",
                    {"type": "yn", "action": "store", "default": True, "metavar": "<y or n>"},
                ),
                (
                    "dothis",
                    {"type": "yn", "action": "store", "default": False, "metavar": "<y or n>"},
                ),
            ]
        )
        self.assertEqual(len(merged), 1)
        self.assertEqual(merged[0][0], "dothis")
        self.assertEqual(merged[0][1]["default"], True)

    def test_merge2(self):
        merged = merge_options(
            [
                (
                    "dothis",
                    {"type": "yn", "action": "store", "default": True, "metavar": "<y or n>"},
                ),
                ("value", {"type": "string", "metavar": "<string>", "short": "v"}),
                (
                    "dothis",
                    {"type": "yn", "action": "store", "default": False, "metavar": "<y or n>"},
                ),
            ]
        )
        self.assertEqual(len(merged), 2)
        self.assertEqual(merged[0][0], "value")
        self.assertEqual(merged[1][0], "dothis")
        self.assertEqual(merged[1][1]["default"], True)


if __name__ == "__main__":
    unittest_main()



# copyright 2003-2011 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This file is part of logilab-common.
#
# logilab-common is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option) any
# later version.
#
# logilab-common is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with logilab-common.  If not, see <http://www.gnu.org/licenses/>.
"""
unit tests for module logilab.common.tree
squeleton generated by /home/syt/bin/py2tests on Jan 20 at 10:43:25
"""

from logilab.common.testlib import TestCase, unittest_main
from logilab.common.tree import (
    Node,
    NodeNotFound,
    post_order_list,
    PostfixedDepthFirstIterator,
    pre_order_list,
    PrefixedDepthFirstIterator,
)

tree = (
    "root",
    (
        ("child_1_1", (("child_2_1", ()), ("child_2_2", (("child_3_1", ()),)))),
        ("child_1_2", (("child_2_3", ()),)),
    ),
)


def make_tree(tuple):
    n = Node(tuple[0])
    for child in tuple[1]:
        n.append(make_tree(child))
    return n


class Node_ClassTest(TestCase):
    """a basic tree node, caracterised by an id"""

    def setUp(self):
        """called before each test from this class"""
        self.o = make_tree(tree)

    def test_flatten(self):
        result = [r.id for r in self.o.flatten()]
        expected = [
            "root",
            "child_1_1",
            "child_2_1",
            "child_2_2",
            "child_3_1",
            "child_1_2",
            "child_2_3",
        ]
        self.assertListEqual(result, expected)

    def test_flatten_with_outlist(self):
        resultnodes = []
        self.o.flatten(resultnodes)
        result = [r.id for r in resultnodes]
        expected = [
            "root",
            "child_1_1",
            "child_2_1",
            "child_2_2",
            "child_3_1",
            "child_1_2",
            "child_2_3",
        ]
        self.assertListEqual(result, expected)

    def test_known_values_remove(self):
        """
        remove a child node
        """
        self.o.remove(self.o.get_node_by_id("child_1_1"))
        self.assertRaises(NodeNotFound, self.o.get_node_by_id, "child_1_1")

    def test_known_values_replace(self):
        """
        replace a child node with another
        """
        self.o.replace(self.o.get_node_by_id("child_1_1"), Node("hoho"))
        self.assertRaises(NodeNotFound, self.o.get_node_by_id, "child_1_1")
        self.assertEqual(self.o.get_node_by_id("hoho"), self.o.children[0])

    def test_known_values_get_sibling(self):
        """
        return the sibling node that has given id
        """
        self.assertEqual(self.o.children[0].get_sibling("child_1_2"), self.o.children[1], None)

    def test_raise_get_sibling_NodeNotFound(self):
        self.assertRaises(NodeNotFound, self.o.children[0].get_sibling, "houhou")

    def test_known_values_get_node_by_id(self):
        """
        return node in whole hierarchy that has given id
        """
        self.assertEqual(self.o.get_node_by_id("child_1_1"), self.o.children[0])

    def test_raise_get_node_by_id_NodeNotFound(self):
        self.assertRaises(NodeNotFound, self.o.get_node_by_id, "houhou")

    def test_known_values_get_child_by_id(self):
        """
        return child of given id
        """
        self.assertEqual(
            self.o.get_child_by_id("child_2_1", recurse=1), self.o.children[0].children[0]
        )

    def test_raise_get_child_by_id_NodeNotFound(self):
        self.assertRaises(NodeNotFound, self.o.get_child_by_id, nid="child_2_1")
        self.assertRaises(NodeNotFound, self.o.get_child_by_id, "houhou")

    def test_known_values_get_child_by_path(self):
        """
        return child of given path (path is a list of ids)
        """
        self.assertEqual(
            self.o.get_child_by_path(["root", "child_1_1", "child_2_1"]),
            self.o.children[0].children[0],
        )

    def test_raise_get_child_by_path_NodeNotFound(self):
        self.assertRaises(NodeNotFound, self.o.get_child_by_path, ["child_1_1", "child_2_11"])

    def test_known_values_depth_down(self):
        """
        return depth of this node in the tree
        """
        self.assertEqual(self.o.depth_down(), 4)
        self.assertEqual(self.o.get_child_by_id("child_2_1", True).depth_down(), 1)

    def test_known_values_depth(self):
        """
        return depth of this node in the tree
        """
        self.assertEqual(self.o.depth(), 0)
        self.assertEqual(self.o.get_child_by_id("child_2_1", True).depth(), 2)

    def test_known_values_width(self):
        """
        return depth of this node in the tree
        """
        self.assertEqual(self.o.width(), 3)
        self.assertEqual(self.o.get_child_by_id("child_2_1", True).width(), 1)

    def test_known_values_root(self):
        """
        return the root node of the tree
        """
        self.assertEqual(self.o.get_child_by_id("child_2_1", True).root(), self.o)

    def test_known_values_leaves(self):
        """
        return a list with all the leaf nodes descendant from this task
        """
        self.assertEqual(
            self.o.leaves(),
            [
                self.o.get_child_by_id("child_2_1", True),
                self.o.get_child_by_id("child_3_1", True),
                self.o.get_child_by_id("child_2_3", True),
            ],
        )

    def test_known_values_lineage(self):
        c31 = self.o.get_child_by_id("child_3_1", True)
        self.assertEqual(
            c31.lineage(),
            [
                self.o.get_child_by_id("child_3_1", True),
                self.o.get_child_by_id("child_2_2", True),
                self.o.get_child_by_id("child_1_1", True),
                self.o,
            ],
        )


class post_order_list_FunctionTest(TestCase):
    def setUp(self):
        """called before each test from this class"""
        self.o = make_tree(tree)

    def test_known_values_post_order_list(self):
        """
        create a list with tree nodes for which the <filter> function returned true
        in a post order foashion
        """
        L = [
            "child_2_1",
            "child_3_1",
            "child_2_2",
            "child_1_1",
            "child_2_3",
            "child_1_2",
            "root",
        ]
        li = [n.id for n in post_order_list(self.o)]
        self.assertEqual(li, L, li)

    def test_known_values_post_order_list2(self):
        """
        create a list with tree nodes for which the <filter> function returned true
        in a post order foashion
        """

        def filter(node):
            if node.id == "child_2_2":
                return 0
            return 1

        L = ["child_2_1", "child_1_1", "child_2_3", "child_1_2", "root"]
        li = [n.id for n in post_order_list(self.o, filter)]
        self.assertEqual(li, L, li)


class PostfixedDepthFirstIterator_ClassTest(TestCase):
    def setUp(self):
        """called before each test from this class"""
        self.o = make_tree(tree)

    def test_known_values_next(self):
        L = ["child_2_1", "child_3_1", "child_2_2", "child_1_1", "child_2_3", "child_1_2", "root"]
        iter = PostfixedDepthFirstIterator(self.o)
        o = next(iter)
        i = 0
        while o:
            self.assertEqual(o.id, L[i])
            o = next(iter)
            i += 1


class pre_order_list_FunctionTest(TestCase):
    def setUp(self):
        """called before each test from this class"""
        self.o = make_tree(tree)

    def test_known_values_pre_order_list(self):
        """
        create a list with tree nodes for which the <filter> function returned true
        in a pre order fashion
        """
        L = [
            "root",
            "child_1_1",
            "child_2_1",
            "child_2_2",
            "child_3_1",
            "child_1_2",
            "child_2_3",
        ]
        li = [n.id for n in pre_order_list(self.o)]
        self.assertEqual(li, L, li)

    def test_known_values_pre_order_list2(self):
        """
        create a list with tree nodes for which the <filter> function returned true
        in a pre order fashion
        """

        def filter(node):
            if node.id == "child_2_2":
                return 0
            return 1

        L = ["root", "child_1_1", "child_2_1", "child_1_2", "child_2_3"]
        li = [n.id for n in pre_order_list(self.o, filter)]
        self.assertEqual(li, L, li)


class PrefixedDepthFirstIterator_ClassTest(TestCase):
    def setUp(self):
        """called before each test from this class"""
        self.o = make_tree(tree)

    def test_known_values_next(self):
        L = ["root", "child_1_1", "child_2_1", "child_2_2", "child_3_1", "child_1_2", "child_2_3"]
        iter = PrefixedDepthFirstIterator(self.o)
        o = next(iter)
        i = 0
        while o:
            self.assertEqual(o.id, L[i])
            o = next(iter)
            i += 1


if __name__ == "__main__":
    unittest_main()



#!/usr/bin/env python
# pylint: disable=W0404,W0622,W0704,W0613,W0152
# copyright 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This file is part of logilab-common.
#
# logilab-common is free software: you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation, either version 2.1 of the License, or (at your
# option) any later version.
#
# logilab-common is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with logilab-common.  If not, see <http://www.gnu.org/licenses/>.
"""Generic Setup script, takes package info from __pkginfo__.py file.
"""
__docformat__ = "restructuredtext en"

from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))

pkginfo = {}
with open(path.join(here, "__pkginfo__.py")) as f:
    exec(f.read(), pkginfo)

# Get the long description from the relevant file
with open(path.join(here, "README.rst"), encoding="utf-8") as f:
    long_description = f.read()

requires = {}
for entry in ("__depends__",):  # "__recommends__"):
    requires.update(pkginfo.get(entry, {}))
install_requires = ["{0} {1}".format(d, v and v or "").strip() for d, v in requires.items()]

setup(
    name=pkginfo["distname"],
    version=pkginfo["version"],
    description=pkginfo["description"],
    long_description=long_description,
    url=pkginfo["web"],
    author=pkginfo["author"],
    author_email=pkginfo["author_email"],
    license=pkginfo["license"],
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=pkginfo["classifiers"],
    packages=find_packages(exclude=["contrib", "docs", "test*"]),
    package_data={"logilab.common": ["py.typed"]},
    include_package_data=True,
    namespace_packages=[pkginfo["subpackage_of"]],
    python_requires=">=3.6",
    install_requires=install_requires,
    tests_require=pkginfo["tests_require"],
    scripts=pkginfo["scripts"],
)



"""unittest_registry data file"""
from logilab.common.registry import yes, RegistrableObject, RegistrableInstance


class Proxy:
    """annoying object should that not be registered, nor cause error"""

    def __getattr__(self, attr):
        return 1


trap = Proxy()


class AppObjectClass(RegistrableObject):
    __registry__ = "zereg"
    __regid__ = "appobject1"
    __select__ = yes()


class AppObjectInstance(RegistrableInstance):
    __registry__ = "zereg"
    __select__ = yes()

    def __init__(self, regid):
        self.__regid__ = regid


appobject2 = AppObjectInstance("appobject2")



# copyright 2003-2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This file is part of logilab-common.
#
# logilab-common is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option) any
# later version.
#
# logilab-common is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with logilab-common.  If not, see <http://www.gnu.org/licenses/>.
"""
unit tests for module modutils (module manipulation utilities)
"""

import doctest
import sys
import warnings

try:
    __file__
except NameError:
    __file__ = sys.argv[0]

from logilab.common.testlib import TestCase, unittest_main
from logilab.common import modutils

from os import path, getcwd, sep
from logilab import common
from logilab.common import tree

warnings.simplefilter("default", DeprecationWarning)
sys.path.insert(0, path.dirname(__file__))
DATADIR = path.join(path.dirname(__file__), "data")


class ModutilsTestCase(TestCase):
    def setUp(self):
        super().setUp()
        self.__common_in_path = common.__path__[0] in sys.path
        if self.__common_in_path:
            sys.path.remove(common.__path__[0])

    def tearDown(self):
        if self.__common_in_path:
            sys.path.insert(0, common.__path__[0])
        super().tearDown()


class ModuleFileTC(ModutilsTestCase):
    package = "mypypa"

    def setUp(self):
        super().setUp()
        for k in list(sys.path_importer_cache.keys()):
            if "MyPyPa" in k:
                del sys.path_importer_cache[k]

    def test_find_zipped_module(self):
        mtype, mfile = modutils._module_file(
            [self.package], [path.join(DATADIR, "MyPyPa-0.1.0.zip")]
        )
        self.assertEqual(mtype, modutils.ZIPFILE)
        self.assertEqual(mfile.split(sep)[-4:], ["test", "data", "MyPyPa-0.1.0.zip", self.package])

    def test_find_egg_module(self):
        mtype, mfile = modutils._module_file(
            [self.package], [path.join(DATADIR, "MyPyPa-0.1.0-py2.5.egg")]
        )
        self.assertEqual(mtype, modutils.ZIPFILE)
        self.assertEqual(
            mfile.split(sep)[-4:], ["test", "data", "MyPyPa-0.1.0-py2.5.egg", self.package]
        )


class load_module_from_name_tc(ModutilsTestCase):
    """load a python module from it's name"""

    def test_knownValues_load_module_from_name_1(self):
        self.assertEqual(modutils.load_module_from_name("sys"), sys)

    def test_knownValues_load_module_from_name_2(self):
        self.assertEqual(modutils.load_module_from_name("os.path"), path)

    def test_raise_load_module_from_name_1(self):
        self.assertRaises(ImportError, modutils.load_module_from_name, "os.path", use_sys=0)


class get_module_part_tc(ModutilsTestCase):
    """given a dotted name return the module part of the name"""

    def test_knownValues_get_module_part_1(self):
        self.assertEqual(
            modutils.get_module_part("logilab.common.modutils"), "logilab.common.modutils"
        )

    def test_knownValues_get_module_part_2(self):
        self.assertEqual(
            modutils.get_module_part("logilab.common.modutils.get_module_part"),
            "logilab.common.modutils",
        )

    def test_knownValues_get_module_part_3(self):
        """relative import from given file"""
        self.assertEqual(
            modutils.get_module_part("interface.Interface", modutils.__file__), "interface"
        )

    def test_knownValues_get_compiled_module_part(self):
        self.assertEqual(modutils.get_module_part("math.log10"), "math")
        self.assertEqual(modutils.get_module_part("math.log10", __file__), "math")

    def test_knownValues_get_builtin_module_part(self):
        self.assertEqual(modutils.get_module_part("sys.path"), "sys")
        self.assertEqual(modutils.get_module_part("sys.path", "__file__"), "sys")

    def test_get_module_part_exception(self):
        self.assertRaises(
            ImportError, modutils.get_module_part, "unknown.module", modutils.__file__
        )


class modpath_from_file_tc(ModutilsTestCase):
    """given an absolute file path return the python module's path as a list"""

    def test_knownValues_modpath_from_file_1(self):
        with warnings.catch_warnings(record=True) as warns:
            self.assertEqual(
                modutils.modpath_from_file(modutils.__file__), ["logilab", "common", "modutils"]
            )
            self.assertIn(
                "[logilab.common.modutils] you should avoid using modpath_from_file()",
                [str(w.message) for w in warns],
            )

    def test_knownValues_modpath_from_file_2(self):
        self.assertEqual(
            modutils.modpath_from_file("unittest_modutils.py", {getcwd(): "arbitrary.pkg"}),
            ["arbitrary", "pkg", "unittest_modutils"],
        )

    def test_raise_modpath_from_file_Exception(self):
        self.assertRaises(Exception, modutils.modpath_from_file, "/turlututu")


class load_module_from_path_tc(ModutilsTestCase):
    def test_do_not_load_twice(self):
        sys.path.insert(0, self.datadir)
        modutils.load_module_from_modpath(["lmfp", "foo"])
        modutils.load_module_from_modpath(["lmfp"])
        self.assertEqual(len(sys.just_once), 1)
        sys.path.pop(0)
        del sys.just_once


class file_from_modpath_tc(ModutilsTestCase):
    """given a mod path (i.e. splited module / package name), return the
    corresponding file, giving priority to source file over precompiled file
    if it exists"""

    def test_site_packages(self):
        from pytz import tzinfo

        self.assertEqual(
            path.realpath(modutils.file_from_modpath(["pytz", "tzinfo"])),
            path.realpath(tzinfo.__file__.replace(".pyc", ".py")),
        )

    def test_std_lib(self):
        from os import path

        self.assertEqual(
            path.realpath(modutils.file_from_modpath(["os", "path"]).replace(".pyc", ".py")),
            path.realpath(path.__file__.replace(".pyc", ".py")),
        )

    def test_xmlplus(self):
        try:
            # don't fail if pyxml isn't installed
            from xml.dom import ext
        except ImportError:
            pass
        else:
            self.assertEqual(
                path.realpath(
                    modutils.file_from_modpath(["xml", "dom", "ext"]).replace(".pyc", ".py")
                ),
                path.realpath(ext.__file__.replace(".pyc", ".py")),
            )

    def test_builtin(self):
        self.assertEqual(modutils.file_from_modpath(["sys"]), None)

    def test_unexisting(self):
        self.assertRaises(ImportError, modutils.file_from_modpath, ["turlututu"])


class get_source_file_tc(ModutilsTestCase):
    def test(self):
        from os import path

        self.assertEqual(
            modutils.get_source_file(path.__file__), path.__file__.replace(".pyc", ".py")
        )

    def test_raise(self):
        self.assertRaises(modutils.NoSourceFile, modutils.get_source_file, "whatever")


class is_standard_module_tc(ModutilsTestCase):
    """
    return true if the module may be considered as a module from the standard
    library
    """

    def test_builtins(self):
        self.assertEqual(modutils.is_standard_module("__builtin__"), False)
        self.assertEqual(modutils.is_standard_module("builtins"), True)

    def test_builtin(self):
        self.assertEqual(modutils.is_standard_module("sys"), True)

    def test_nonstandard(self):
        self.assertEqual(modutils.is_standard_module("logilab"), False)

    def test_unknown(self):
        self.assertEqual(modutils.is_standard_module("unknown"), False)

    def test_4(self):
        self.assertEqual(modutils.is_standard_module("marshal"), True)
        self.assertEqual(modutils.is_standard_module("pickle"), True)
        self.assertEqual(modutils.is_standard_module("email"), True)
        self.assertEqual(modutils.is_standard_module("StringIO"), sys.version_info < (3, 0))
        venv_py3 = sys.version_info[0] >= 3 and hasattr(sys, "real_prefix")
        if not venv_py3:
            # those modules are symlinked by virtualenv (but not by python's venv)
            self.assertEqual(modutils.is_standard_module("hashlib"), True)
            self.assertEqual(modutils.is_standard_module("io"), True)

    def test_custom_path(self):
        self.assertEqual(modutils.is_standard_module("data.module", (DATADIR,)), True)
        self.assertEqual(modutils.is_standard_module("data.module", (path.abspath(DATADIR),)), True)

    def test_failing_border_cases(self):
        # using a subpackage/submodule path as std_path argument
        self.assertEqual(modutils.is_standard_module("logilab.common", common.__path__), False)
        # using a module + object name as modname argument
        self.assertEqual(modutils.is_standard_module("sys.path"), True)
        # this is because only the first package/module is considered
        self.assertEqual(modutils.is_standard_module("sys.whatever"), True)
        self.assertEqual(modutils.is_standard_module("logilab.whatever", common.__path__), False)


class is_relative_tc(ModutilsTestCase):
    def test_knownValues_is_relative_1(self):
        self.assertEqual(modutils.is_relative("modutils", common.__path__[0]), True)

    def test_knownValues_is_relative_2(self):
        self.assertEqual(modutils.is_relative("modutils", tree.__file__), True)

    def test_knownValues_is_relative_3(self):
        self.assertEqual(modutils.is_relative("logilab.common.modutils", common.__path__[0]), False)


class get_modules_tc(ModutilsTestCase):
    def test_knownValues_get_modules_1(self):  # XXXFIXME: TOWRITE
        """given a directory return a list of all available python modules, even
        in subdirectories
        """
        import data.find_test as data

        mod_path = ("data", "find_test")
        modules = sorted(modutils.get_modules(path.join(*mod_path), data.__path__[0]))
        self.assertSetEqual(
            set(modules),
            {
                ".".join(mod_path + (mod,))
                for mod in ("module", "module2", "noendingnewline", "nonregr")
            },
        )


class get_modules_files_tc(ModutilsTestCase):
    def test_knownValues_get_module_files_1(self):  # XXXFIXME: TOWRITE
        """given a directory return a list of all available python module's files, even
        in subdirectories
        """
        import data

        modules = sorted(
            modutils.get_module_files(path.join(DATADIR, "find_test"), data.__path__[0])
        )
        self.assertEqual(
            modules,
            [
                path.join(DATADIR, "find_test", x)
                for x in [
                    "__init__.py",
                    "module.py",
                    "module2.py",
                    "noendingnewline.py",
                    "nonregr.py",
                ]
            ],
        )

    def test_load_module_set_attribute(self):
        import logilab.common.fileutils
        import logilab

        del logilab.common.fileutils
        del sys.modules["logilab.common.fileutils"]
        m = modutils.load_module_from_modpath(["logilab", "common", "fileutils"])
        self.assertTrue(hasattr(logilab, "common"))
        self.assertTrue(hasattr(logilab.common, "fileutils"))
        self.assertTrue(m is logilab.common.fileutils)


def load_tests(loader, tests, ignore):
    from logilab.common import modutils

    tests.addTests(doctest.DocTestSuite(modutils))
    return tests


if __name__ == "__main__":
    unittest_main()
